@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -55,6 +55,7 @@ use warnings;
 use base qw(RT::Action::SendEmail);
 
 use Email::Address;
+use Regexp::Common;
 
 =head2 Prepare
 
@@ -71,8 +72,8 @@ sub Prepare {
 
 =head2 SetRecipients
 
-Sets the recipients of this meesage to Owner, Requestor, AdminCc, Cc or All. 
-Explicitly B<does not> notify the creator of the transaction by default
+Sets the recipients of this message to Owner, Requestor, AdminCc, Cc or All.
+Explicitly B<does not> notify the creator of the transaction by default.
 
 =cut
 
@@ -82,16 +83,77 @@ sub SetRecipients {
     my $ticket = $self->TicketObj;
 
     my $arg = $self->Argument;
-    $arg =~ s/\bAll\b/Owner,Requestor,AdminCc,Cc/;
 
     my ( @To, @PseudoTo, @Cc, @Bcc );
 
+    my %args = map { $_ => 1 } $self->SplitArgument;
 
-    if ( $arg =~ /\bRequestor\b/ ) {
+    if ( $args{All} ) {
+        $args{$_} ||= 1 for qw/Owner Requestor AdminCc Cc/;
+    }
+
+    if ( $args{Requestor} ) {
         push @To, $ticket->Requestors->MemberEmailAddresses;
     }
 
-    if ( $arg =~ /\bCc\b/ ) {
+    # custom role syntax:   gives:
+    #   name                  (role name,  Cc)
+    #   RT::CustomRole-#      (role with id, Cc)
+    #   name/To               (role name,  To)
+    #   RT::CustomRole-#/To   (role with id, To)
+    #   name/Cc               (role name,  Cc)
+    #   RT::CustomRole-#/Cc   (role with id, Cc)
+    #   name/Bcc              (role name,  Bcc)
+    #   RT::CustomRole-#/Bcc  (role with id, Bcc)
+
+    # this has to happen early because adding To addresses affects how Cc
+    # is handled
+
+    for my $item ( sort keys %args ) {
+        next if $item =~ /^(?:All|Owner|Requestor|AdminCc|Cc|OtherRecipients|AlwaysNotifyActor|NeverNotifyActor)$/;
+        my ( $name, $type ) = ( $item =~ m{^(.+?)(?:/(To|Cc|Bcc))?$} );
+        next unless $name;
+
+        my $role;
+        if ( $name =~ /^RT::CustomRole-(\d+)$/ ) {
+            my $id = $1;
+            $role = RT::CustomRole->new( $self->CurrentUser );
+            $role->Load( $id );
+        }
+        else {
+            my $roles = RT::CustomRoles->new( $self->CurrentUser );
+            $roles->Limit( FIELD => 'Name', VALUE => $name, CASESENSITIVE => 0 );
+
+            # custom roles are named uniquely, but just in case there are
+            # multiple matches, bail out as we don't know which one to use
+            $role = $roles->First;
+            if ( $role ) {
+                $role = undef if $roles->Next;
+            }
+        }
+
+        unless ($role && $role->id) {
+            $RT::Logger->debug("Unable to load custom role from scrip action argument '$item'");
+            next;
+        }
+
+        my @role_members = (
+            $ticket->RoleGroup($role->GroupType)->MemberEmailAddresses,
+            $ticket->QueueObj->RoleGroup($role->GroupType)->MemberEmailAddresses,
+        );
+
+        if (!$type || $type eq 'Cc') {
+            push @Cc, @role_members;
+        }
+        elsif ($type eq 'Bcc') {
+            push @Bcc, @role_members;
+        }
+        elsif ($type eq 'To') {
+            push @To, @role_members;
+        }
+    }
+
+    if ( $args{Cc} ) {
 
         #If we have a To, make the Ccs, Ccs, otherwise, promote them to To
         if (@To) {
@@ -104,9 +166,10 @@ sub SetRecipients {
         }
     }
 
-    if (   $arg =~ /\bOwner\b/
+    if (   $args{Owner}
         && $ticket->OwnerObj->id != RT->Nobody->id
         && $ticket->OwnerObj->EmailAddress
+        && not $ticket->OwnerObj->Disabled
     ) {
         # If we're not sending to Ccs or requestors,
         # then the Owner can be the To.
@@ -119,7 +182,7 @@ sub SetRecipients {
 
     }
 
-    if ( $arg =~ /\bAdminCc\b/ ) {
+    if ( $args{AdminCc} ) {
         push ( @Bcc, $ticket->AdminCc->MemberEmailAddresses  );
         push ( @Bcc, $ticket->QueueObj->AdminCc->MemberEmailAddresses  );
     }
@@ -131,27 +194,12 @@ sub SetRecipients {
         }
     }
 
-    my $creatorObj = $self->TransactionObj->CreatorObj;
-    my $creator = $creatorObj->EmailAddress() || '';
-
-    #Strip the sender out of the To, Cc and AdminCc and set the 
-    # recipients fields used to build the message by the superclass.
-    # unless a flag is set 
-    my $TransactionCurrentUser = RT::CurrentUser->new;
-    $TransactionCurrentUser->LoadByName($creatorObj->Name);
-    if (RT->Config->Get('NotifyActor',$TransactionCurrentUser)) {
-        @{ $self->{'To'} }  = @To;
-        @{ $self->{'Cc'} }  = @Cc;
-        @{ $self->{'Bcc'} } = @Bcc;
-    }
-    else {
-        @{ $self->{'To'} }  = grep ( lc $_ ne lc $creator, @To );
-        @{ $self->{'Cc'} }  = grep ( lc $_ ne lc $creator, @Cc );
-        @{ $self->{'Bcc'} } = grep ( lc $_ ne lc $creator, @Bcc );
-    }
+    @{ $self->{'To'} }       = @To;
+    @{ $self->{'Cc'} }       = @Cc;
+    @{ $self->{'Bcc'} }      = @Bcc;
     @{ $self->{'PseudoTo'} } = @PseudoTo;
 
-    if ( $arg =~ /\bOtherRecipients\b/ ) {
+    if ( $args{OtherRecipients} ) {
         if ( my $attachment = $self->TransactionObj->Attachments->First ) {
             push @{ $self->{'NoSquelch'}{'Cc'} ||= [] }, map $_->address,
                 Email::Address->parse( $attachment->GetHeader('RT-Send-Cc') );
@@ -161,6 +209,72 @@ sub SetRecipients {
     }
 }
 
+=head2 RemoveInappropriateRecipients
+
+Remove transaction creator as appropriate for the NotifyActor setting.
+
+To send email to the selected recipients regardless of RT's NotifyActor
+configuration, include AlwaysNotifyActor in the list of arguments. Or to
+always suppress email to the selected recipients regardless of RT's
+NotifyActor configuration, include NeverNotifyActor in the list of arguments.
+
+=cut
+
+sub RemoveInappropriateRecipients {
+    my $self = shift;
+
+    my $creatorObj = $self->TransactionObj->CreatorObj;
+    my $creator = $creatorObj->EmailAddress() || '';
+    my $TransactionCurrentUser = RT::CurrentUser->new;
+    $TransactionCurrentUser->LoadByName($creatorObj->Name);
+
+    my %args = map { $_ => 1 } $self->SplitArgument;
+
+    $self->RecipientFilter(
+        Callback => sub {
+            return unless lc $_[0] eq lc $creator;
+            return "not sending to $creator, creator of the transaction, due to NotifyActor setting";
+        },
+    ) if $args{NeverNotifyActor} ||
+         (!RT->Config->Get('NotifyActor',$TransactionCurrentUser)
+         && !$args{AlwaysNotifyActor});
+
+    $self->SUPER::RemoveInappropriateRecipients();
+}
+
+
+=head2 SplitArgument
+
+Split comma separated argument. Like CSV, it also supports quoted
+values, so values like "'foo, bar'" is treated like a single value.
+
+Return the list of the split values.
+
+=cut
+
+sub SplitArgument {
+    my $self = shift;
+    my $arg  = shift // $self->Argument;
+
+    return unless defined $arg && length $arg;
+
+    $arg =~ s!^\s+!!;
+    $arg =~ s!\s+$!!;
+
+    my @args;
+    while ( $arg =~ s/^($RE{quoted}|[^,]+)(?:,\s*|$)//g ) {
+        my $item = $1;
+        if ( $item =~ /^(['"])(.*)\1/ ) {
+            next unless length $2;
+            push @args, $2;
+        }
+        else {
+            push @args, $item;
+        }
+    }
+    return @args;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
