@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2018 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -190,6 +190,14 @@ sub Prepare {
         # utf-8 here is for _FindOrGuessCharset in I18N.pm
         # it's not the final charset/encoding sent
         $part->head->mime_attr( "Content-Type.charset" => 'utf-8' );
+
+        # Set proper transfer encoding to prevent long lines in
+        # body(1000+ chars) that are not allowed according to RFC821.
+        # Some mail servers automatically insert "!" into long lines to
+        # indicate this incompatibility.
+        if ( !$part->head->mime_attr('Content-Transfer-Encoding') ) {
+            $part->head->mime_attr( 'Content-Transfer-Encoding' => $part->suggest_encoding );
+        }
     }
 
     RT::I18N::SetMIMEEntityToEncoding(
@@ -542,7 +550,8 @@ sub AddTicket {
     }
     if ( $ticket_mime->parts ) {
         my $email_mime = $self->TemplateObj->MIMEObj;
-        $email_mime->make_multipart;
+        $email_mime->make_multipart( 'mixed', Force => 1 )
+            unless $email_mime->effective_type eq 'multipart/mixed';
         $email_mime->add_part($ticket_mime);
     }
     return;
