@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2018 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -561,6 +561,18 @@ sub IsMessageContentType {
     return $self->ContentType =~ m{^\s*message/}i ? 1 : 0;
 }
 
+=head2 IsAttachmentContentDisposition
+
+Returns a boolean indicating if the Content-Disposition of this attachment
+is a MIME attachment.
+
+=cut
+
+sub IsAttachmentContentDisposition {
+    my $self = shift;
+    return ( $self->GetHeader('Content-Disposition') // '' ) =~ m{^\s*attachment\b}i ? 1 : 0;
+}
+
 =head2 Addresses
 
 Returns a hashref of all addresses related to this attachment.
@@ -637,6 +649,14 @@ This is not protection using quoted printable or base64 encoding.
 sub EncodedHeaders {
     my $self = shift;
     my $encoding = shift || 'utf8';
+
+    # Require Encode::HanExtra to handle more encodings it supports.
+    # The regex is based on the names documented in Encode::HanExtra.
+    if ( $encoding =~ /^(?:big5(?:-1984|-2003|ext|plus)|cccii|cns11643-[1-7f]|euc-tw|gb18030|unisys(?:-sosi(?:1|2))?)$/ ) {
+        unless ( Encode::HanExtra->require ) {
+            RT->Logger->error("Need Encode::HanExtra to handle $encoding");
+        }
+    }
     return Encode::encode( $encoding, $self->Headers );
 }
 
@@ -743,6 +763,96 @@ sub SetHeader {
     $self->__Set( Field => 'Headers', Value => $newheader);
 }
 
+=head2 ReplaceHeaders ( Search => 'SEARCH', Replacement => 'Replacement' )
+
+Search the attachments table's Header column for the search string provided.
+When a match is found call the SetHeader() method on the header with the match,
+either set the header to empty or a replacement value.
+
+=cut
+
+sub ReplaceHeaders {
+    my $self = shift;
+    my %args = (
+        Search      => undef,
+        Replacement => '',
+        @_,
+    );
+
+    return ( 0, $self->loc('No Search string provided') ) unless $args{Search};
+
+    my $updated;
+    foreach my $header ( $self->SplitHeaders ) {
+        my ( $tag, $value ) = split /:/, $header, 2;
+        if ( $value =~ s/\Q$args{Search}\E/$args{Replacement}/ig ) {
+            my ( $ret, $msg ) = $self->SetHeader( $tag, $value );
+            if ( $ret ) {
+                $updated ||= 1;
+            }
+            else {
+                RT::Logger->error("Could not set header: $tag to $value: $msg");
+                return ( $ret, $msg );
+            }
+        }
+    }
+
+    if ( $updated ) {
+        return ( 1, $self->loc('Headers cleared') );
+    }
+    else {
+        return ( 0, $self->loc('No header matches found') );
+    }
+}
+
+=head2 ReplaceContent ( Search => 'SEARCH', Replacement => 'Replacement' )
+
+Search the attachments table's Content column for the search string provided.
+When a match is found either replace it with the provided replacement string or an
+empty string.
+
+=cut
+
+sub ReplaceContent {
+    my $self = shift;
+    my %args = (
+        Search      => undef,
+        Replacement => '',
+        @_,
+    );
+
+    return ( 0, $self->loc('No search string provided') ) unless $args{Search};
+
+    my $content = $self->Content;
+
+    if ( $content && $content =~ s/\Q$args{Search}\E/$args{Replacement}/ig ) {
+        my ( $encoding, $encoded_content, undef, undef, $note_args )
+          = $self->_EncodeLOB( Encode::encode( 'UTF-8', $content ) );
+
+        $RT::Handle->BeginTransaction;
+        if ($note_args) {
+            $self->TransactionObj->Object->_NewTransaction(%$note_args);
+        }
+
+        my ( $ret, $msg ) = $self->_Set( Field => 'Content', Value => $encoded_content );
+        unless ($ret) {
+            $RT::Handle->Rollback;
+            return ( $ret, $msg );
+        }
+
+        if ( ( $self->ContentEncoding // '' ) ne $encoding ) {
+            my ( $ret, $msg ) = $self->_Set( Field => 'ContentEncoding', Value => $encoding );
+            unless ($ret) {
+                $RT::Handle->Rollback;
+                return ( $ret, $msg );
+            }
+        }
+        $RT::Handle->Commit;
+        return ( $ret, 'Content replaced' );
+    }
+    return ( 0, $self->loc('No content matches found') );
+}
+
+
 sub _CanonicalizeHeaderValue {
     my $self  = shift;
     my $value = shift;
@@ -793,6 +903,47 @@ sub _SplitHeaders {
     return(@headers);
 }
 
+=head2 CryptStatus
+
+Returns the parsed status from the X-RT-GnuPG-Status or
+X-RT-SMIME-Status header.
+
+The return value is an array of hashrefs; each hashref is as described
+in L<RT::Crypt::ParseStatus>; however, each hashref has one additional
+entry 'Protocol' which is the name of the crypto protocol used and is
+one of 'SMIME' or 'GnuPG'.
+
+If no crypto header exists, returns an empty array
+
+=cut
+
+sub CryptStatus {
+    my $self = shift;
+    my @ret  = ();
+
+    foreach my $h ( $self->SplitHeaders ) {
+        next unless $h =~ /^X-RT-(GnuPG|SMIME)-Status:/i;
+        my $protocol = $1;
+        my ( $h_key, $h_val ) = split( /:\s*/, $h, 2 );
+        my @result = RT::Crypt->ParseStatus(
+            Protocol => $protocol,
+            Status   => $h_val
+        );
+
+        # Canonicalize protocol case so it's always SMIME or GnuPG
+        if ( uc($protocol) eq 'SMIME' ) {
+            $protocol = 'SMIME';
+        }
+        elsif ( uc($protocol) eq 'GNUPG' ) {
+            $protocol = 'GnuPG';
+        }
+        foreach my $hash (@result) {
+            $hash->{'Protocol'} = $protocol;
+            push( @ret, $hash );
+        }
+    }
+    return @ret;
+}
 
 sub Encrypt {
     my $self = shift;
@@ -821,9 +972,9 @@ sub Encrypt {
         RT->Config->Get('CorrespondAddress'),
         RT->Config->Get('CommentAddress'),
     ) {
-        my %res = RT::Crypt->GetKeysInfo( Key => $address, Type => 'private' );
+        my %res = RT::Crypt->GetKeysInfo( Queue => $queue, Key => $address, Type => 'private' );
         next if $res{'exit_code'} || !$res{'info'};
-        %res = RT::Crypt->GetKeysForEncryption( $address );
+        %res = RT::Crypt->GetKeysForEncryption( Queue => $queue, Recipient => $address );
         next if $res{'exit_code'} || !$res{'info'};
         $encrypt_for = $address;
     }
@@ -833,6 +984,7 @@ sub Encrypt {
 
     my $content = $self->Content;
     my %res = RT::Crypt->SignEncryptContent(
+        Queue => $queue,
         Content => \$content,
         Sign => 0,
         Encrypt => 1,
@@ -937,7 +1089,7 @@ sub _CacheConfig {
 =head2 id
 
 Returns the current value of id.
-(In the database, id is stored as int(11).)
+(In the database, id is stored as int(19).)
 
 
 =cut
@@ -946,7 +1098,7 @@ Returns the current value of id.
 =head2 TransactionId
 
 Returns the current value of TransactionId.
-(In the database, TransactionId is stored as int(11).)
+(In the database, TransactionId is stored as int(19).)
 
 
 
@@ -955,7 +1107,7 @@ Returns the current value of TransactionId.
 
 Set TransactionId to VALUE.
 Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
-(In the database, TransactionId will be stored as a int(11).)
+(In the database, TransactionId will be stored as a int(19).)
 
 
 =cut
@@ -964,7 +1116,7 @@ Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
 =head2 Parent
 
 Returns the current value of Parent.
-(In the database, Parent is stored as int(11).)
+(In the database, Parent is stored as int(19).)
 
 
 
@@ -973,7 +1125,7 @@ Returns the current value of Parent.
 
 Set Parent to VALUE.
 Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
-(In the database, Parent will be stored as a int(11).)
+(In the database, Parent will be stored as a int(19).)
 
 
 =cut
@@ -1128,11 +1280,11 @@ sub _CoreAccessible {
     {
 
         id =>
-                {read => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => ''},
+                {read => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(19)', default => ''},
         TransactionId =>
-                {read => 1, write => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => ''},
+                {read => 1, write => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(19)', default => ''},
         Parent =>
-                {read => 1, write => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => '0'},
+                {read => 1, write => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(19)', default => '0'},
         MessageId =>
                 {read => 1, write => 1, sql_type => 12, length => 160,  is_blob => 0,  is_numeric => 0,  type => 'varchar(160)', default => ''},
         Subject =>
