@@ -621,6 +621,16 @@ our %META;
             $config->Set( InlineEditPanelBehavior => %$behavior );
         },
     },
+    ShowSearchNavigation => {
+        Section     => 'Ticket display',
+        Overridable => 1,
+        SortOrder   => 13,
+        Widget      => '/Widgets/Form/Boolean',
+        WidgetArguments => {
+            Description => 'Show search navigation', # loc
+            Hints       => 'Show search navigation links of "First", "Last", "Prev" and "Next"', # loc
+        }
+    },
 
     # User overridable locale options
     DateTimeFormat => {
@@ -920,12 +930,35 @@ our %META;
                 $opt->{'Incoming'} = \@enabled;
             }
             if ( $opt->{'Outgoing'} ) {
-                if (not $enabled{$opt->{'Outgoing'}}) {
-                    $RT::Logger->warning($opt->{'Outgoing'}.
+                if (ref($opt->{'Outgoing'}) eq 'HASH') {
+                    # Check each entry in the hash
+                    foreach my $q (keys(%{$opt->{'Outgoing'}})) {
+                        if (not $enabled{$opt->{'Outgoing'}->{$q}}) {
+                            if ($q ne '') {
+                                $RT::Logger->warning($opt->{'Outgoing'}->{$q}.
+                                                     " explicitly set as outgoing Crypt plugin for queue $q, but not marked Enabled; "
+                                                     . (@enabled ? "using $enabled[0]" : "removing"));
+                            } else {
+                                $RT::Logger->warning($opt->{'Outgoing'}->{$q}.
+                                                     " explicitly set as default outgoing Crypt plugin, but not marked Enabled; "
+                                                     . (@enabled ? "using $enabled[0]" : "removing"));
+                            }
+                            $opt->{'Outgoing'}->{$q} = $enabled[0];
+                        }
+                    }
+                    # If there's no entry for the default queue, set one
+                    if (!$opt->{'Outgoing'}->{''} && scalar(@enabled)) {
+                        $RT::Logger->warning("No default outgoing Crypt plugin set; using $enabled[0]");
+                        $opt->{'Outgoing'}->{''} = $enabled[0];
+                    }
+                } else {
+                    if (not $enabled{$opt->{'Outgoing'}}) {
+                        $RT::Logger->warning($opt->{'Outgoing'}.
                                              " explicitly set as outgoing Crypt plugin, but not marked Enabled; "
                                              . (@enabled ? "using $enabled[0]" : "removing"));
+                    }
+                    $opt->{'Outgoing'} = $enabled[0] unless $enabled{$opt->{'Outgoing'}};
                 }
-                $opt->{'Outgoing'} = $enabled[0] unless $enabled{$opt->{'Outgoing'}};
             } else {
                 $opt->{'Outgoing'} = $enabled[0];
             }
@@ -970,6 +1003,13 @@ our %META;
                     delete $opt->{CAPath};
                 }
             }
+
+            if ($opt->{CheckCRL} && ! RT::Crypt::SMIME->SupportsCRLfile) {
+                $opt->{CheckCRL} = 0;
+                $RT::Logger->warn(
+                    "Your version of OpenSSL does not support the -CRLfile option; disabling \$SMIME{CheckCRL}"
+                );
+            }
         },
     },
     GnuPG        => {
@@ -1499,6 +1539,9 @@ our %META;
     AssetHideSimpleSearch => {
         Widget => '/Widgets/Form/Boolean',
     },
+    AssetMultipleOwner => {
+        Widget => '/Widgets/Form/Boolean',
+    },
     AssetShowSearchResultCount => {
         Widget => '/Widgets/Form/Boolean',
     },
@@ -1839,6 +1882,9 @@ our %META;
     LogoURL => {
         Widget => '/Widgets/Form/String',
     },
+    LogoutURL => {
+        Widget => '/Widgets/Form/String',
+    },
     OwnerEmail => {
         Widget => '/Widgets/Form/String',
     },
@@ -1848,6 +1894,9 @@ our %META;
     RedistributeAutoGeneratedMessages => {
         Widget => '/Widgets/Form/String',
     },
+    RTSupportEmail => {
+        Widget => '/Widgets/Form/String',
+    },
     SelfServiceRequestUpdateQueue => {
         Widget => '/Widgets/Form/String',
     },
@@ -1930,6 +1979,75 @@ our %META;
         Widget => '/Widgets/Form/Select',
         WidgetArguments => { Values => [qw(ASC DESC)] },
     },
+    SelfServiceUserPrefs => {
+        Widget          => '/Widgets/Form/Select',
+        WidgetArguments => {
+            Values      => [qw(edit-prefs view-info edit-prefs-view-info full-edit)],
+            ValuesLabel => {
+                'edit-prefs'           => 'Edit Locale and change password',                           # loc
+                'view-info'            => 'View all the info',                                         # loc
+                'edit-prefs-view-info' => 'View all the info, and edit Locale and change password',    # loc
+                'full-edit'            => 'View and update all the info',                              # loc
+            },
+        },
+    },
+    AssetDefaultSearchResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    AssetSimpleSearchFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    AssetSummaryFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    AssetSummaryRelatedTicketsFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    DefaultSearchResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    DefaultSelfServiceSearchResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    GroupSearchResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    GroupSummaryExtraInfo => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    GroupSummaryTicketListFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    LDAPFilter => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    LDAPGroupFilter => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    MoreAboutRequestorExtraInfo => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    MoreAboutRequestorTicketListFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    UserDataResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    UserSearchResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    UserSummaryExtraInfo => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    UserSummaryTicketListFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    UserTicketDataResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
+    UserTransactionDataResultFormat => {
+        Widget => '/Widgets/Form/MultilineString',
+    },
     LogToSyslogConf => {
         Immutable     => 1,
     },
@@ -2094,7 +2212,15 @@ sub _LoadConfig {
         }
     };
     if ($@) {
-        return 1 if $is_site && $@ =~ /^Can't locate \Q$args{File}/;
+
+        if ( $is_site && $@ =~ /^Can't locate \Q$args{File}/ ) {
+
+            # Since perl 5.18, the "Can't locate ..." error message contains
+            # more details. warn to help debug if there is a permission issue.
+            warn qq{Couldn't load RT config file $args{'File'}:\n\n$@} if $@ =~ /Permission denied at/;
+            return 1;
+        }
+
         if ( $is_site || $@ !~ /^Can't locate \Q$args{File}/ ) {
             die qq{Couldn't load RT config file $args{'File'}:\n\n$@};
         }
@@ -2688,6 +2814,7 @@ sub RefreshConfigFromDatabase {
     my $needs_update = RT->System->ConfigCacheNeedsUpdate;
     if ($needs_update > $database_config_cache_time) {
         $self->LoadConfigFromDatabase();
+        $HTML::Mason::Commands::ReloadScrubber = 1;
         $database_config_cache_time = $needs_update;
     }
 }
@@ -2721,7 +2848,9 @@ sub LoadConfigFromDatabase {
         if ($meta->{'Source'}) {
             my %source = %{ $meta->{'Source'} };
             if ($source{'SiteConfig'} && $source{'File'} ne 'database') {
-                warn("Change of config option '$name' at $source{File} line $source{Line} has been overridden by the config setting from the database. Please remove it from $source{File} or from the database to avoid confusion.");
+                push @PreInitLoggerMessages,
+                    "Change of config option '$name' at $source{File} line $source{Line} has been overridden by the config setting from the database. "
+                    ."Please remove it from $source{File} or from the database to avoid confusion.";
             }
         }
 
@@ -2772,6 +2901,11 @@ sub LoadConfigFromDatabase {
     }
 }
 
+sub _GetFromFilesOnly {
+    my ( $self, $name ) = @_;
+    return $original_setting_from_files{$name} ? $original_setting_from_files{$name}[0] : undef;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
