@@ -332,8 +332,7 @@ sub CallGnuPG {
         %GnuPGOptions,
         %{ $args{Options} || {} },
     );
-    my $gnupg = GnuPG::Interface->new;
-    $gnupg->call( $self->GnuPGPath );
+    my $gnupg = GnuPG::Interface->new( call => $self->GnuPGPath );
     $gnupg->options->hash_init(
         _PrepareGnuPGOptions( %opt ),
     );
@@ -1491,6 +1490,15 @@ sub ParseStatus {
 
             foreach my $line ( @status[ $i .. $#status ] ) {
                 next unless $line =~ /^VALIDSIG\s+(.*)/;
+                # Fingerprint = key fingerprint in hex
+                # CreationDate = key creation date (YYYY-MM-DD)
+                # Timestamp = signature creation time (seconds from UNIX epoch)
+                # ExpireTimestamp = signature expiration time (since epoch) or 0 for "never expires"
+                # Version = signature version straight from the packet
+                # PubkeyAlgo = Public key algorithm (https://tools.ietf.org/html/rfc4880#section-9.1)
+                # HashAlgo = Hash algorithm (https://tools.ietf.org/html/rfc4880#section-9.4)
+                # Class = Signature type (https://tools.ietf.org/html/rfc4880#section-5.2.1)
+                # PKFingerprint = Primary Key Fingerprint
                 @res{ qw(
                     Fingerprint
                     CreationDate
@@ -1504,6 +1512,8 @@ sub ParseStatus {
                     PKFingerprint
                     Other
                 ) } = split /\s+/, $1, 10;
+                $res{HashAlgoName} = $self->HashAlgorithmToName($res{HashAlgo}) if defined($res{HashAlgo});
+                $res{PubkeyAlgoName} = $self->PubkeyAlgorithmToName($res{PubkeyAlgo}) if defined($res{PubkeyAlgo});
                 last;
             }
             push @res, \%res;
@@ -1688,7 +1698,7 @@ sub GetKeysInfo {
     for my $key (@{$res{info}}) {
         $key->{Formatted} =
             join("; ", map {$_->{String}} @{$key->{User}})
-                . " (".substr($key->{Fingerprint}, -8) . ")";
+                . " (" . $key->{Fingerprint} . ")";
     }
 
     return %res;
@@ -1713,6 +1723,7 @@ sub ParseKeysInfo {
                 Empty Empty Capabilities Other
             ) } = split /:/, $line, 12;
 
+            $info{AlgorithmName} = $self->PubkeyAlgorithmToName($info{Algorithm}) if defined($info{Algorithm});
             # workaround gnupg's wierd behaviour, --list-keys command report calculated trust levels
             # for any model except 'always', so you can change models and see changes, but not for 'always'
             # we try to handle it in a simple way - we set ultimate trust for any key with trust
@@ -1740,6 +1751,7 @@ sub ParseKeysInfo {
                 Created Expire Empty OwnerTrustChar
                 Empty Empty Capabilities Other
             ) } = split /:/, $line, 12;
+            $info{AlgorithmName} = $self->PubkeyAlgorithmToName($info{Algorithm}) if defined($info{Algorithm});
             @info{qw(OwnerTrust OwnerTrustTerse OwnerTrustLevel)} = 
                 _ConvertTrustChar( $info{'OwnerTrustChar'} );
             $info{ $_ } = $self->ParseDate( $info{ $_ } )
@@ -1856,6 +1868,68 @@ sub ImportKey {
     );
 }
 
+sub ReceiveKey {
+    my $self = shift;
+    my $key  = shift;
+
+    return $self->CallGnuPG(
+        Command     => "recv_keys",
+        CommandArgs => [ '--', $key ],
+    );
+}
+
+sub TrustKey {
+    my $self  = shift;
+    my $key   = shift;
+    my $level = shift;
+
+    $level++; # the level format in --import-ownertrust input is +1
+
+    return $self->CallGnuPG(
+        Command => "--import-ownertrust",
+        Content => "$key:$level:\n",
+    );
+}
+
+sub SearchKey {
+    my $self = shift;
+    my $key  = shift;
+
+    my @output;
+    my %ret = $self->CallGnuPG(
+        Command     => "search_keys",
+        CommandArgs => [ '--', $key ],
+        Output      => \@output,
+        # gpg hangs if command handle is supplied :/
+        Handles     => { command => 0 },
+    );
+
+    my @results;
+    my $result;
+    for my $line ( @output ) {
+        if ( $line =~ /^\(\d+\)/ ) {
+            push @results, { Summary => $result } if $result;
+            $result = $line;
+        }
+        else {
+            $result .= $line;
+        }
+    }
+    push @results, { Summary => $result } if $result;
+
+    for my $item ( @results ) {
+        if ( $item->{Summary} =~ /^\s*\d+ bit \w+ key (\w{8,})/mi ) {
+            $item->{Key} = $1;
+        }
+        else {
+            RT->Logger->warning("Couldn't find key from gpg search result: $item->{Summary}");
+        }
+    }
+    @results = grep { $_->{Key} } @results;
+
+    return ( %ret, results => \@results );
+}
+
 sub GnuPGPath {
     state $cache = RT->Config->Get('GnuPG')->{'GnuPG'};
     $cache = $_[1] if @_ > 1;
@@ -1864,7 +1938,6 @@ sub GnuPGPath {
 
 sub Probe {
     my $self = shift;
-    my $gnupg = GnuPG::Interface->new;
 
     my $bin = $self->GnuPGPath();
     unless ($bin) {
@@ -1894,7 +1967,7 @@ sub Probe {
         $self->GnuPGPath( $bin = $path );
     }
 
-    $gnupg->call( $bin );
+    my $gnupg = GnuPG::Interface->new( call => $bin );
     $gnupg->options->hash_init(
         _PrepareGnuPGOptions( RT->Config->Get('GnuPGOptions') )
     );
@@ -1956,10 +2029,65 @@ sub _make_gpg_handles {
         foreach grep !defined $handle_map{$_}, 
         qw(stdin stdout stderr logger status command);
 
+    for my $type ( keys %handle_map ) {
+        delete $handle_map{$type} if !$handle_map{$type};
+    }
+
     my $handles = GnuPG::Handles->new(%handle_map);
     return ($handles, \%handle_map);
 }
 
+# Gigne a PGP hash algorithm number, return the algorithm name.
+# See https://tools.ietf.org/html/rfc4880#section-9.4
+sub HashAlgorithmToName {
+    my ( $self, $alg ) = @_;
+    return 'MD5'         if ( $alg == 1 );
+    return 'SHA-1'       if ( $alg == 2 );
+    return 'RIPE-MD/160' if ( $alg == 3 );
+    return 'Reserved'    if ( $alg >= 4 && $alg <= 7 );
+    return 'SHA256'      if ( $alg == 8 );
+    return 'SHA384'      if ( $alg == 9 );
+    return 'SHA512'      if ( $alg == 10 );
+    return 'SHA224'      if ( $alg == 11 );
+    return undef;
+}
+
+# Given a PGP public-key algorithm number, return the algorithm name.
+# See https://tools.ietf.org/html/rfc4880#section-9.1
+sub PubkeyAlgorithmToName {
+    my ( $self, $alg ) = @_;
+    return 'RSA'                         if ( $alg == 1 );
+    return 'RSA Encrypt-Only'            if ( $alg == 2 );
+    return 'RSA Sign-Only'               if ( $alg == 3 );
+    return 'Elgamal Encrypt-Ony'         if ( $alg == 16 );
+    return 'DSA'                         if ( $alg == 17 );
+    return 'Reserved for EC'             if ( $alg == 18 );
+    return 'Reserved for ECDSA'          if ( $alg == 19 );
+    return 'Reserved (formerly Elgamal)' if ( $alg == 20 );
+    return 'Reserved (DH)'               if ( $alg == 21 );
+    return undef;
+}
+
+# Given a public-key fingerprint, return the public key (in PEM format)
+# if we have it, undef if we do not.
+sub GetPubkey {
+    my ( $self, $fingerprint ) = @_;
+
+    # Sanity-check the fingerprint, which must be a string of
+    # hex digits
+    if ( $fingerprint =~ /[^0-9a-fA-F]/ ) {
+        return undef;
+    }
+
+    my @pubkey;
+    my %res = $self->CallGnuPG(
+        Command     => 'export_keys',
+        CommandArgs => $fingerprint,
+        Output      => \@pubkey
+    );
+    return join( '', @pubkey );
+}
+
 RT::Base->_ImportOverlays();
 
 1;
