@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -61,6 +61,11 @@ use IPC::Run3 0.036 'run3';
 use RT::Util 'safe_run_child';
 use Crypt::X509;
 use String::ShellQuote 'shell_quote';
+use LWP;
+
+# This will be set to a true value by Probe
+# if "openssl verify" supports the -CRLfile option
+our $OpenSSL_Supports_CRLfile;
 
 =head1 NAME
 
@@ -75,12 +80,20 @@ You should start from reading L<RT::Crypt>.
     Set( %SMIME,
         Enable => 1,
         OpenSSL => '/usr/bin/openssl',
-        Keyring => '/opt/rt4/var/data/smime',
-        CAPath  => '/opt/rt4/var/data/smime/signing-ca.pem',
+        Keyring => '/opt/rt5/var/data/smime',
+        CAPath  => '/opt/rt5/var/data/smime/signing-ca.pem',
         Passphrase => {
             'queue.address@example.com' => 'passphrase',
+            'another.queue.address@example.com' => {
+                Encryption => 'passphrase for encryption certificate',
+                Signing    => 'passphrase for signing certificate',
+            },
             '' => 'fallback',
         },
+        OtherCertificatesToSend => '/opt/rt5/var/data/smime/other-certs.pem',
+        CheckCRL => 0,
+        CheckOCSP => 0,
+        CheckRevocationDownloadTimeout => 30,
     );
 
 =head3 OpenSSL
@@ -119,6 +132,33 @@ C<Passphrase> may be set to a scalar (to use for all keys), an anonymous
 function, or a hash (to look up by address).  If the hash is used, the
 '' key is used as a default.
 
+=head3 OtherCertificatesToSend
+
+C<OtherCertificatesToSend> is a path to a PEM-formatted certificate file.
+Certificates in the file will be include in outgoing signed emails.
+
+Depending on use cases, you might need to include a chain of certificates so
+receiving agents can verify. CA could also be included here.
+
+=head3 CheckCRL
+
+A boolean option that determines whether or not we attempt to check if
+a certificate is revoked by downloading a CRL.  The default value is
+false (do not check).  Additionally, if AcceptUntrustedCAs is true, RT
+will I<never> download a CRL or check an OCSP URL for a certificate
+signed by an untrusted CA.
+
+=head3 CheckOCSP
+
+A boolean option that determines whether or not we check if a certificate
+is revoked by checking the OCSP URL (if any).  The default value is
+false.
+
+=head3 CheckRevocationDownloadTimeout
+
+Timeout in seconds for downloading a CRL or issuer certificate for
+OCSP checking.  The default is 30 seconds.
+
 =head2 Keyring configuration
 
 RT looks for keys in the directory configured in the L</Keyring> option
@@ -127,6 +167,11 @@ on users, private SSL keys are only loaded from disk.  Keys and
 certificates should be concatenated, in in PEM format, in files named
 C<email.address@example.com.pem>, for example.
 
+For addresses that have separate certificates for encryption/decryption
+and signing, the PEM files need to be named like
+C<email.address@example.com.encryption.pem> and
+C<email.address@example.com.signing.pem>, respectively.
+
 These files need be readable by the web server user which is running
 RT's web interface; however, if you are running cronjobs or other
 utilities that access RT directly via API, and may generate
@@ -184,6 +229,14 @@ sub Probe {
             \$buf, \$err
         ) };
 
+        if ($err && $err =~ /Invalid command/) {
+            ($buf, $err) = ('', '');
+            safe_run_child { run3( [$bin, "list", "-commands"],
+                \undef,
+                \$buf, \$err
+            ) };
+        }
+
         if ($? or $err) {
             $RT::Logger->warning(
                 "RT's SMIME libraries couldn't successfully execute openssl.".
@@ -195,6 +248,14 @@ sub Probe {
                     " SMIME support has been disabled");
             return;
         } else {
+            ($buf, $err) = ('', '');
+            # Interrogate openssl verify command to see if it supports
+            # the -CRLfile option.
+            safe_run_child { run3( [$bin, 'verify', '-help'],
+                                   \undef, \$buf, \$err) };
+            if ($err =~ /-CRLfile/) {
+                $OpenSSL_Supports_CRLfile = 1;
+            }
             return 1;
         }
     }
@@ -208,6 +269,7 @@ sub SignEncrypt {
         Sign => 1,
         Signer => undef,
         Passphrase => undef,
+        OtherCertificatesToSend => undef,
 
         Encrypt => 1,
         Recipients => undef,
@@ -272,6 +334,7 @@ sub _SignEncrypt {
         Sign => 1,
         Signer => undef,
         Passphrase => undef,
+        OtherCertificatesToSend => undef,
 
         Encrypt => 1,
         Recipients => [],
@@ -288,7 +351,7 @@ sub _SignEncrypt {
         foreach my $address ( @addresses ) {
             $RT::Logger->debug( "Considering encrypting message to " . $address );
 
-            my %key_info = $self->GetKeysInfo( Key => $address );
+            my %key_info = $self->GetKeysInfo( Key => $address, For => 'Encryption' );
             unless ( defined $key_info{'info'} ) {
                 $res{'exit_code'} = 1;
                 my $reason = 'Key not found';
@@ -329,7 +392,7 @@ sub _SignEncrypt {
 
     my @commands;
     if ( $args{'Sign'} ) {
-        my $file = $self->CheckKeyring( Key => $args{'Signer'} );
+        my $file = $self->CheckKeyring( Key => $args{'Signer'}, For => 'Signing' );
         unless ($file) {
             $res{'status'} .= $self->FormatStatus({
                 Operation => "KeyCheck", Status => "MISSING",
@@ -340,13 +403,15 @@ sub _SignEncrypt {
             $res{exit_code} = 1;
             return (undef, %res);
         }
-        $args{'Passphrase'} = $self->GetPassphrase( Address => $args{'Signer'} )
+        $args{'Passphrase'} = $self->GetPassphrase( Address => $args{'Signer'}, For => 'Signing' )
             unless defined $args{'Passphrase'};
 
+        $args{OtherCertificatesToSend} //= $opts->{OtherCertificatesToSend};
         push @commands, [
             $self->OpenSSLPath, qw(smime -sign),
             -signer => $file,
             -inkey  => $file,
+            $args{OtherCertificatesToSend} ? ( -certfile => $args{OtherCertificatesToSend} ) : (),
             (defined $args{'Passphrase'} && length $args{'Passphrase'})
                 ? (qw(-passin env:SMIME_PASS))
                 : (),
@@ -468,14 +533,23 @@ sub Verify {
             $res{'status'} = $self->FormatStatus({
                 Operation => "Verify", Status => "BAD",
                 Message => "The signing CA was not trusted",
+                Serial => $signer->{'Serial Number'} || 'unknown',
                 UserString => $signer->{User}[0]{String},
+                ExpireTimestamp => $signer->{Expire}->Unix(),
+                CreatedTimestamp => $signer->{Created}->Unix(),
                 Trust => "NONE",
             });
             return %res;
         }
 
+        my ($address) = Email::Address->parse($signer->{User}[0]{String});
+        last unless $address;
         my $user = RT::User->new( $RT::SystemUser );
-        $user->LoadOrCreateByEmail( $signer->{User}[0]{String} );
+        $user->LoadOrCreateByEmail(
+            EmailAddress => $address->address,
+            RealName     => $address->phrase,
+            Comments     => 'Autocreated during SMIME parsing',
+        );
         my $current_key = $user->SMIMECertificate;
         last if $current_key && $current_key eq $key;
 
@@ -494,6 +568,9 @@ sub Verify {
         $res{'status'} = $self->FormatStatus({
             Operation => "Verify", Status => "DONE",
             Message => "The signature is good, unknown signer",
+            ExpireTimestamp => $signer->{Expire}->Unix(),
+            CreatedTimestamp => $signer->{Created}->Unix(),
+            Serial => $signer->{'Serial Number'} || 'unknown',
             Trust => "UNKNOWN",
         });
         return %res;
@@ -507,9 +584,13 @@ sub Verify {
 
     $res{'status'} = $self->FormatStatus({
         Operation => "Verify", Status => "DONE",
-        Message => "The signature is good, signed by ".$signer->{User}[0]{String}.", trust is ".$signer->{TrustTerse},
+        Message => "The signature is good, signed by ".$signer->{User}[0]{String}.", assured by " . $signer->{Issuer}[0]{String} . ", trust is ".$signer->{TrustTerse},
+        Serial => $signer->{'Serial Number'} || 'unknown',
         UserString => $signer->{User}[0]{String},
         Trust => uc($signer->{TrustTerse}),
+        Issuer => $signer->{Issuer}[0]{String},
+        ExpireTimestamp => $signer->{Expire}->Unix(),
+        CreatedTimestamp => $signer->{Created}->Unix(),
     });
 
     return %res;
@@ -569,14 +650,14 @@ sub _Decrypt {
     my ($buf, $encrypted_to, %res);
 
     foreach my $address ( @addresses ) {
-        my $file = $self->CheckKeyring( Key => $address );
+        my $file = $self->CheckKeyring( Key => $address, For => 'Encryption' );
         unless ( $file ) {
             my $keyring = RT->Config->Get('SMIME')->{'Keyring'};
             $RT::Logger->debug("No key found for $address in $keyring directory");
             next;
         }
 
-        local $ENV{SMIME_PASS} = $self->GetPassphrase( Address => $address );
+        local $ENV{SMIME_PASS} = $self->GetPassphrase( Address => $address, For => 'Encryption' );
         local $SIG{CHLD} = 'DEFAULT';
         my $cmd = [
             $self->OpenSSLPath,
@@ -789,13 +870,37 @@ sub CheckIfProtected {
 sub GetKeysForEncryption {
     my $self = shift;
     my %args = (Recipient => undef, @_);
-    return $self->GetKeysInfo( Key => delete $args{'Recipient'}, %args, Type => 'public' );
+    my $recipient = delete $args{'Recipient'};
+    my %res = $self->GetKeysInfo( Key => $recipient, %args, Type => 'public', For => 'Encryption' );
+    return %res unless $res{'info'};
+
+    foreach my $key ( splice @{ $res{'info'} } ) {
+        if ( not $key->{'Expire'} ) {
+            # we continue here as it's most probably a problem with the key,
+            # so later during encryption we'll get verbose errors
+            $RT::Logger->error(
+                "Trying to send an encrypted message to ". $recipient
+                .", but we couldn't get expiration date of the key."
+            );
+        }
+        elsif ( $key->{'Expire'}->Diff( time ) < 0 ) {
+            $RT::Logger->info(
+                "Trying to send an encrypted message to ". $recipient
+                .", but ignoring expired key " . $key->{Fingerprint}
+            );
+            next;
+        }
+        push @{ $res{'info'} }, $key;
+    }
+    delete $res{'info'} unless @{ $res{'info'} };
+
+    return %res;
 }
 
 sub GetKeysForSigning {
     my $self = shift;
     my %args = (Signer => undef, @_);
-    return $self->GetKeysInfo( Key => delete $args{'Signer'}, %args, Type => 'private' );
+    return $self->GetKeysInfo( Key => delete $args{'Signer'}, %args, Type => 'private', For => 'Signing' );
 }
 
 sub GetKeysInfo {
@@ -804,6 +909,7 @@ sub GetKeysInfo {
         Key   => undef,
         Type  => 'public',
         Force => 0,
+        For   => undef,
         @_
     );
 
@@ -820,7 +926,7 @@ sub GetKeysInfo {
 
 sub GetKeyContent {
     my $self = shift;
-    my %args = ( Key => undef, @_ );
+    my %args = ( Key => undef, For => undef, @_ );
 
     my $key;
     if ( my $file = $self->CheckKeyring( %args ) ) {
@@ -841,11 +947,17 @@ sub CheckKeyring {
     my $self = shift;
     my %args = (
         Key => undef,
+        For => undef,
         @_,
     );
     my $keyring = RT->Config->Get('SMIME')->{'Keyring'};
     return undef unless $keyring;
 
+    if ( $args{For} ) {
+        my $file = File::Spec->catfile( $keyring, $args{'Key'} . '.' . lc( $args{For} ) . '.pem' );
+        return $file if -f $file;
+    }
+
     my $file = File::Spec->catfile( $keyring, $args{'Key'} .'.pem' );
     return undef unless -f $file;
 
@@ -883,8 +995,18 @@ sub GetCertificateInfo {
             my $method = $type . "_" . $USER_MAP{$_};
             $data{$_} = $cert->$method if $cert->can($method);
         }
-        $data{String} = Email::Address->new( @data{'Name', 'EmailAddress'} )->format
-            if $data{EmailAddress};
+
+        # Use the correct procedure as per
+        # https://tools.ietf.org/html/rfc5750#section-3
+        # to extract the subject's email address
+        if ($type eq 'subject') {
+            $data{EmailAddress} = $self->ExtractSubjectEmailAddress($cert);
+        }
+        if ($data{EmailAddress}) {
+            $data{String} = Email::Address->new( @data{'Name', 'EmailAddress'} )->format;
+        } else {
+            $data{String} = $data{Name};
+        }
         return \%data;
     };
 
@@ -907,6 +1029,61 @@ sub GetCertificateInfo {
         stderr => ''
     );
 
+    # First, check if the certificate verifies without checking
+    # revocation status
+    $self->RunOpenSSLVerify($PEM, \%res);
+
+    if ($res{info}[0]{TrustLevel} != 2) {
+        # Not signed by trusted CA; return
+        return %res;
+    }
+
+    # If we're not configured to check CRLs or OCSP, just return
+    # what we have.
+    return %res unless (RT::Config->Get('SMIME')->{'CheckCRL'} ||
+                        RT::Config->Get('SMIME')->{'CheckOCSP'}   );
+
+    # Check if certificate has been revoked using OCSP if the cert has
+    # an OCSP URL.  Unfortunately, Crypt::X509 doesn't let us query
+    # for OCSP URLs, so we need to run OpenSSL.
+    if (RT::Config->Get('SMIME')->{'CheckOCSP'}) {
+        my $ocsp_result = $self->CheckRevocationUsingOCSP($PEM, \%res);
+        if ($ocsp_result) {
+            # We got a definitive result from OCSP; return
+            return %res;
+        }
+    }
+
+    # OCSP didn't give us a result, or was disabled  Try downloading CRL.
+    if (RT::Config->Get('SMIME')->{'CheckCRL'}) {
+        if ($OpenSSL_Supports_CRLfile) {
+            # We fetch the CRL file ourselves using LWP rather than
+            # using OpenSSL's -crl_download option so we can
+            # control the timeout.
+            my ($url) = @{$cert->CRLDistributionPoints || []};
+            if ($url) {
+                my $crl_file = $self->DownloadAndConvertCRLToPEM($url);
+                if ($crl_file) {
+                    $self->RunOpenSSLVerify($PEM, \%res, '-crl_check', '-CRLfile', $crl_file);
+                } else {
+                    $res{info}[0]{Trust} .= " (NOTE: Unable to download CRL)";
+                }
+            }
+        }
+    }
+
+    return %res;
+}
+
+sub RunOpenSSLVerify
+{
+    my $self = shift;
+    my $PEM = shift;
+    my $res = shift;
+    # Remaining args are extra arguments to "openssl verify"
+
+    $res->{stderr} = '';
+
     # Check the validity
     my $ca = RT->Config->Get('SMIME')->{'CAPath'};
     if ($ca) {
@@ -918,39 +1095,230 @@ sub GetCertificateInfo {
         }
 
         local $SIG{CHLD} = 'DEFAULT';
+
         my $cmd = [
             $self->OpenSSLPath,
-            'verify', @ca_verify,
-        ];
+            'verify', @ca_verify, @_,
+          ];
         my $buf = '';
-        safe_run_child { run3( $cmd, \$PEM, \$buf, \$res{stderr} ) };
+        safe_run_child { run3( $cmd, \$PEM, \$buf, \$res->{stderr} ) };
 
         if ($buf =~ /^stdin: OK$/) {
-            $res{info}[0]{Trust} = "Signed by trusted CA $res{info}[0]{Issuer}[0]{String}";
-            $res{info}[0]{TrustTerse} = "full";
-            $res{info}[0]{TrustLevel} = 2;
+            $res->{info}[0]{Trust} = "Signed by trusted CA $res->{info}[0]{Issuer}[0]{String}";
+            $res->{info}[0]{TrustTerse} = "full";
+            $res->{info}[0]{TrustLevel} = 2;
+            $res->{exit_code} = 0;
         } elsif ($? == 0 or ($? >> 8) == 2) {
-            $res{info}[0]{Trust} = "UNTRUSTED signing CA $res{info}[0]{Issuer}[0]{String}";
-            $res{info}[0]{TrustTerse} = "none";
-            $res{info}[0]{TrustLevel} = -1;
+            if ($res->{stderr} =~ /certificate revoked/i) {
+                $res->{info}[0]{Trust} = "REVOKED certificate from CA $res->{info}[0]{Issuer}[0]{String}";
+                $res->{info}[0]{TrustTerse} = "none (revoked certificate)";
+            } else {
+                $res->{info}[0]{Trust} = "UNTRUSTED signing CA $res->{info}[0]{Issuer}[0]{String}";
+                $res->{info}[0]{TrustTerse} = "none";
+            }
+            $res->{info}[0]{TrustLevel} = -1;
+            $res->{exit_code} = $?;
         } else {
-            $res{exit_code} = $?;
-            $res{message} = "openssl exited with error code ". ($? >> 8)
+            $res->{exit_code} = $?;
+            $res->{message} = "openssl exited with error code ". ($? >> 8)
                 ." and stout: $buf";
-            $res{info}[0]{Trust} = "unknown (openssl failed)";
-            $res{info}[0]{TrustTerse} = "unknown";
-            $res{info}[0]{TrustLevel} = 0;
+            $res->{info}[0]{Trust} = "unknown (openssl failed)";
+            $res->{info}[0]{TrustTerse} = "unknown";
+            $res->{info}[0]{TrustLevel} = 0;
         }
     } else {
-        $res{info}[0]{Trust} = "unknown (no CAPath set)";
-        $res{info}[0]{TrustTerse} = "unknown";
-        $res{info}[0]{TrustLevel} = 0;
+        $res->{info}[0]{Trust} = "unknown (no CAPath set)";
+        $res->{info}[0]{TrustTerse} = "unknown";
+        $res->{info}[0]{TrustLevel} = 0;
+    }
+    $res->{info}[0]{Formatted} = $res->{info}[0]{User}[0]{String} . " (issued by $res->{info}[0]{Issuer}[0]{String})";
+}
+
+# Extract the subject email address from an S/MIME certificate.
+# https://tools.ietf.org/html/rfc5750#section-3
+sub ExtractSubjectEmailAddress {
+    my $self = shift;
+    my $cert = shift;
+
+    # 1: Check SubjectAltName
+    # "The email address SHOULD be in the subjectAltName extension"
+
+    my $altnames = $cert->SubjectAltName;
+    if ( $altnames && ( ref($altnames) eq 'ARRAY' ) ) {
+
+        # Pick the first email address from the array.
+        foreach my $alt (@$altnames) {
+            if ( $alt =~ s/^rfc822name=//i ) {
+                return $alt;
+            }
+        }
     }
 
-    $res{info}[0]{Formatted} = $res{info}[0]{User}[0]{String}
-        . " (issued by $res{info}[0]{Issuer}[0]{String})";
+    # 2: Check subject_email
+    my $ret = $cert->subject_email;
+    return $ret if ( defined($ret) && ( $ret ne '' ) );
 
-    return %res;
+    # 3: If the subject_cn looks like an email address,
+    # return that
+    my $email_address;
+    eval { ($email_address) = Email::Address->parse( $cert->subject_cn ); };
+    if ($email_address) {
+        return $email_address->address;
+    }
+
+    # No sensible email address found
+    return undef;
+}
+
+sub DownloadAndConvertCRLToPEM {
+    my ($self, $url) = @_;
+    my $tmpdir = File::Temp::tempdir( TMPDIR => 1, CLEANUP => 1 );
+    my $ua = LWP::UserAgent->new(env_proxy => 1);
+    $ua->timeout(RT::Config->Get('SMIME')->{CheckRevocationDownloadTimeout});
+
+    my $resp = $ua->get($url);
+    return undef unless $resp->is_success;
+
+    my $fname = File::Spec->catfile($tmpdir, 'crl.pem');
+    my $in = $resp->decoded_content;
+    if ($in !~ /-----BEGIN X509 CRL-----/) {
+        $in =  "-----BEGIN X509 CRL-----\n" .
+                MIME::Base64::encode_base64($in) .
+               "-----END X509 CRL-----\n";
+    }
+    if ( open my $fh, '>', $fname ) {
+        print $fh $in;
+        close($fh);
+        return $fname;
+    }
+    return undef;
+}
+
+# Returns: 1 if cert has been revoked, 0 if it has definitely NOT been revoked,
+# undef if OCSP check failed
+sub CheckRevocationUsingOCSP {
+    my ($self, $PEM, $res) = @_;
+
+    # Can't do anything without a CAPath
+    my $ca = RT->Config->Get('SMIME')->{'CAPath'};
+    return undef unless $ca;
+
+    my ($out, $err);
+    $out = '';
+    $err = '';
+    # We need to download the issuer certificate, so look for its URL and
+    # that of the OCSP
+    safe_run_child { run3( [$self->OpenSSLPath, 'x509', '-noout', '-text'],
+                           \$PEM, \$out, \$err ) };
+    return undef unless $out =~ /CA Issuers - URI:(https?:.*)/;
+    my $issuer_url = $1;
+
+    return undef unless $out =~ /OCSP - URI:(https?:.*)/;
+    my $ocsp_url = $1;
+
+    # We have the issuer certificate URL; make a temp dir and grab it
+    my $tmpdir = File::Temp::tempdir( TMPDIR => 1, CLEANUP => 1 );
+    my $issuer = File::Spec->catfile($tmpdir, 'issuer.crt');
+    my $ua = LWP::UserAgent->new(env_proxy => 1);
+    $ua->timeout(RT::Config->Get('SMIME')->{CheckRevocationDownloadTimeout});
+
+    my $resp = $ua->get($issuer_url);
+    return undef unless $resp->is_success;
+
+    open(my $fh, '>', $issuer) or return undef;
+    my $content = $resp->decoded_content;
+    if ($content !~ /BEGIN CERTIFICATE/) {
+        # Convert from DER to PEM
+        $content = "-----BEGIN CERTIFICATE-----\n" .
+            MIME::Base64::encode_base64($content) .
+            "-----END CERTIFICATE-----\n";
+    }
+    print $fh $content;
+    close($fh);
+
+    # Check for revocation
+    my @ca_verify;
+    if (-d $ca) {
+        @ca_verify = ('-CApath', $ca);
+    } elsif (-f $ca) {
+        @ca_verify = ('-CAfile', $ca);
+    }
+    $out = '';
+    $err = '';
+
+    safe_run_child { run3( [$self->OpenSSLPath(), 'ocsp', '-issuer', $issuer, '-cert', '-', @ca_verify, '-url', $ocsp_url],
+                           \$PEM, \$out, \$err) };
+    return undef unless $? == 0;
+
+    if ($out =~ /^-: revoked/) {
+        $res->{info}[0]{Trust} = "REVOKED certificate checked against OCSP URI $ocsp_url";
+        $res->{info}[0]{TrustTerse} = "none (revoked certificate)";
+        $res->{info}[0]{TrustLevel} = -1;
+        $res->{exit_code} = 0;
+        return 1;
+    }
+    if ($out =~ /^-: good/) {
+        # Definitely NOT revoked.  Return 0, but not undef
+        return 0;
+    }
+
+    return undef;
+}
+
+# Accessor function to query if OpenSSL supports -CRLfile
+# without having to know a package variable name.
+sub SupportsCRLfile {
+    return $OpenSSL_Supports_CRLfile;
+};
+
+# Given a Transaction object, find the application/x-rt-original-message
+# (if any).  If found, try to find the S/MIME signature.  Return
+# the certificate found in that signature.
+# Returns: undef if we could not find a certificate; otherwise, an
+# S/MIME certificate in PEM format.
+sub GetCertificateForTransaction {
+    my ($self, $txn) = @_;
+
+    my $attachments = $txn->Attachments;
+    my $found;
+
+    # Look for the application/x-rt-original-message attachment
+    while (my $a = $attachments->Next) {
+        if (lc($a->ContentType) eq 'application/x-rt-original-message') {
+            $found = $a;
+            last;
+        }
+    }
+    return undef unless $found;
+
+    my $str = $found->Content;
+    return undef unless $str;
+
+    my $tmpdir = File::Temp::tempdir( TMPDIR => 1, CLEANUP => 1 );
+    my $p = MIME::Parser->new();
+    $p->output_dir($tmpdir);
+    my $entity = $p->parse_data($str);
+    return undef unless $entity;
+
+    # Now look for application/pkcs7-signature
+    $found = undef;
+    foreach my $part ($entity->parts_DFS) {
+        if (lc($part->mime_type) eq 'application/pkcs7-signature') {
+            $found = $part;
+            last;
+        }
+    }
+    return undef unless $found;
+    return undef unless $found->bodyhandle;
+
+    # Feed to openssl and extract the certificate
+    my $pkcs7 = $found->bodyhandle->as_string;
+    my $out = '';
+    my $err = '';
+    safe_run_child { run3( [$self->OpenSSLPath, 'pkcs7', '-inform', 'DER', '-print_certs' ],
+                           \$pkcs7, \$out, \$err ) };
+    return undef unless $out =~ /-----BEGIN CERTIFICATE-----/;
+    return $out;
 }
 
 1;
