@@ -172,6 +172,29 @@ sub ApplySortOrder {
     } );
 }
 
+=head2 LimitToNotAdded
+
+Takes either list of object ids or nothing. Limits collection
+to custom roles to listed objects or any corespondingly.
+
+=cut
+
+sub LimitToNotAdded {
+    my $self = shift;
+    return RT::ObjectCustomRoles->new( $self->CurrentUser )->LimitTargetToNotAdded( $self => @_ );
+}
+
+=head2 LimitToAdded
+
+Limits collection to custom roles to listed objects or any corespondingly.
+
+=cut
+
+sub LimitToAdded {
+    my $self = shift;
+    return RT::ObjectCustomRoles->new( $self->CurrentUser )->LimitTargetToAdded( $self => @_ );
+}
+
 RT::Base->_ImportOverlays();
 
 1;
