@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -171,10 +171,15 @@ sub Set {
     if ( $format eq 'unix' ) {
         return $self->Unix( $args{'Value'} );
     }
-    elsif (
-        ($format eq 'sql' || $format eq 'iso')
-        && $args{'Value'} =~ /^(\d{4})-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d)$/
-    ) {
+    elsif ($format eq 'sql' && $args{'Value'} =~ /^(\d{4})-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d)$/) {
+        local $@;
+        my $u = eval { Time::Local::timegm($6, $5, $4, $3, $2-1, $1) } || 0;
+        $RT::Logger->warning("Invalid date $args{'Value'}: $@") if $@ && !$u;
+        return $self->Unix( $u > 0 ? $u : 0 );
+    }
+    elsif ( $format eq 'iso' &&
+          ( $args{'Value'} =~ /^(\d{4})-(\d\d)-(\d\d)[ T](\d\d):(\d\d):(\d\d)Z?$/ ||
+            $args{'Value'} =~ /^(\d{4})(\d\d)(\d\d)T(\d\d)(\d\d)(\d\d)Z?$/)) {
         local $@;
         my $u = eval { Time::Local::timegm($6, $5, $4, $3, $2-1, $1) } || 0;
         $RT::Logger->warning("Invalid date $args{'Value'}: $@") if $@ && !$u;
@@ -213,35 +218,19 @@ sub Set {
         }
     }
     elsif ( $format eq 'unknown' ) {
-        require Time::ParseDate;
-        # the module supports only legacy timezones like PDT or EST...
-        # so we parse date as GMT and later apply offset, this only
-        # should be applied to absolute times, so compensate shift in NOW
-        my $now = time;
-        $now += ($self->Localtime( $args{Timezone}, $now ))[9];
-        my ($date, $error) = Time::ParseDate::parsedate(
-            $args{'Value'},
-            GMT           => 1,
-            NOW           => $now,
-            UK            => RT->Config->Get('DateDayBeforeMonth'),
-            PREFER_PAST   => RT->Config->Get('AmbiguousDayInPast'),
-            PREFER_FUTURE => RT->Config->Get('AmbiguousDayInFuture'),
-        );
-        unless ( defined $date ) {
-            $RT::Logger->warning(
-                "Couldn't parse date '$args{'Value'}' by Time::ParseDate"
+        if ( RT->Config->Get('PreferDateTimeFormatNatural') ) {
+            return $self->Unix(
+                   $self->ParseByDateTimeFormatNatural(%args)
+                || $self->ParseByTimeParseDate(%args)
+                || 0
+            );
+        } else {
+            return $self->Unix(
+                   $self->ParseByTimeParseDate(%args)
+                || $self->ParseByDateTimeFormatNatural(%args)
+                || 0
             );
-            return $self->Unix(0);
         }
-
-        # apply timezone offset
-        $date -= ($self->Localtime( $args{Timezone}, $date ))[9];
-
-        $RT::Logger->debug(
-            "RT::Date used Time::ParseDate to make '$args{'Value'}' $date\n"
-        );
-
-        return $self->Unix($date || 0);
     }
     else {
         $RT::Logger->error(
@@ -253,6 +242,78 @@ sub Set {
     return $self->Unix;
 }
 
+=head2 ParseByTimeParseDate
+
+Parse date using Time::ParseDate.
+return undef if it fails to parse, otherwise return epoch time.
+
+=cut
+
+sub ParseByTimeParseDate {
+    my $self = shift;
+    my %args = @_;
+    require Time::ParseDate;
+    # the module supports only legacy timezones like PDT or EST...
+    # so we parse date as GMT and later apply offset, this only
+    # should be applied to absolute times, so compensate shift in NOW
+    my $now = time;
+    $now += ($self->Localtime( $args{Timezone}, $now ))[9];
+    my ($date, $error) = Time::ParseDate::parsedate(
+        $args{'Value'},
+        GMT           => 1,
+        NOW           => $now,
+        UK            => RT->Config->Get('DateDayBeforeMonth'),
+        PREFER_PAST   => RT->Config->Get('AmbiguousDayInPast'),
+        PREFER_FUTURE => RT->Config->Get('AmbiguousDayInFuture'),
+    );
+    unless ( defined $date ) {
+        $RT::Logger->warning(
+            "Couldn't parse date '$args{'Value'}' by Time::ParseDate"
+        );
+        return undef;
+    }
+
+    # apply timezone offset
+    $date -= ($self->Localtime( $args{Timezone}, $date ))[9];
+
+    $RT::Logger->debug(
+        "RT::Date used Time::ParseDate to make '$args{'Value'}' $date\n"
+    );
+    return $date;
+}
+
+=head2 ParseByDateTimeFormatNatural
+
+Parse date using DateTime::Format::Natural.
+return undef if it fails to parse, otherwise return epoch time.
+
+=cut
+
+sub ParseByDateTimeFormatNatural {
+    my $self = shift;
+    my %args = @_;
+    require DateTime::Format::Natural;
+
+    my $parser = DateTime::Format::Natural->new(
+        prefer_future => RT->Config->Get('AmbiguousDayInPast') ? 0 : RT->Config->Get('AmbiguousDayInFuture'),
+        time_zone     => $self->Timezone($args{Timezone}),
+    );
+    my ($dt) = eval { $parser->parse_datetime($args{Value}) };
+    if ( !$@ && $parser->success && $dt ) {
+        my $date = $dt->epoch;
+        $RT::Logger->debug(
+            "RT::Date used DateTime::Format::Natural to make '$args{'Value'}' $date\n"
+        );
+        return $date;
+    }
+    else {
+        $RT::Logger->warning(
+            "Couldn't parse date '$args{'Value'}' by DateTime::Format::Natural"
+        );
+        return undef;
+    }
+}
+
 =head2 SetToNow
 
 Set the object's time to the current time. Takes no arguments
@@ -355,6 +416,20 @@ most vague variant.
 Turn on short notation with one character units, for example
 "3M 2d 1m 10s".
 
+=item * MinUnit
+
+The min unit to use. Default is 'second'.
+
+=item * MaxUnit
+
+The max unit to use. Default is 'year'.
+
+=item * Unit
+
+The unit to use. This implies showing "< ..." or "> ..." when the value
+exceeds boundary. E.g. when it's set to "hour", 24*3600*2 seconds will show
+as "> 24 hours" instead of "2 days" or "48 hours".
+
 =back
 
 =cut
@@ -362,9 +437,9 @@ Turn on short notation with one character units, for example
 sub DurationAsString {
     my $self     = shift;
     my $duration = int shift;
-    my %args = ( Show => 1, Short => 0, @_ );
+    my %args = ( Show => 1, Short => 0, MinUnit => 'second', MaxUnit => 'year', @_ );
 
-    unless ( $duration ) {
+    unless ( $duration || $args{Unit} ) {
         return $args{Short}? $self->loc("0s") : $self->loc("0 seconds");
     }
 
@@ -376,40 +451,177 @@ sub DurationAsString {
 
     my $coef = 2;
     my $i = 0;
+
+    my %skip;
+    my @units = qw/second minute hour day week month year/;
+    for my $unit (@units) {
+        last if $args{MinUnit} =~ /$unit/i;
+        $skip{$unit} = 1;
+    }
+
+    if ( $args{Unit} ) {
+        my ( $locstr, $unit );
+        if ( $args{Unit} eq 'minute' ) {
+            if ( $duration < $MINUTE ) {
+                if ( $args{Short} ) {
+                    $locstr = '< [_1]m';    # loc
+                }
+                else {
+                    $locstr = '< [_1] minute';    # loc
+                }
+                $unit = 1;
+            }
+            elsif ( $duration > $coef * $HOUR ) {
+                if ( $args{Short} ) {
+                    $locstr = '> [_1]m';          # loc
+                }
+                else {
+                    $locstr = '> [_1] minutes';    # loc
+                }
+                $unit = $coef * 60;
+            }
+        }
+        elsif ( $args{Unit} eq 'hour' ) {
+            if ( $duration < $HOUR ) {
+                if ( $args{Short} ) {
+                    $locstr = '< [_1]h';           # loc
+                }
+                else {
+                    $locstr = '< [_1] hour';       # loc
+                }
+                $unit = 1;
+            }
+            elsif ( $duration > $coef * $DAY ) {
+                if ( $args{Short} ) {
+                    $locstr = '> [_1]h';           # loc
+                }
+                else {
+                    $locstr = '> [_1] hours';      # loc
+                }
+                $unit = $coef * 24;
+            }
+        }
+        elsif ( $args{Unit} eq 'day' ) {
+            if ( $duration < $DAY ) {
+                if ( $args{Short} ) {
+                    $locstr = '< [_1]d';           # loc
+                }
+                else {
+                    $locstr = '< [_1] day';        # loc
+                }
+                $unit = 1;
+            }
+            elsif ( $duration > $coef * $WEEK ) {
+                if ( $args{Short} ) {
+                    $locstr = '> [_1]d';           # loc
+                }
+                else {
+                    $locstr = '> [_1] days';       # loc
+                }
+                $unit = $coef * 7;
+            }
+        }
+        elsif ( $args{Unit} eq 'week' ) {
+            if ( $duration < $WEEK ) {
+                if ( $args{Short} ) {
+                    $locstr = '< [_1]W';           # loc
+                }
+                else {
+                    $locstr = '< [_1] week';       # loc
+                }
+                $unit = 1;
+            }
+            elsif ( $duration > $coef * 4 * $WEEK ) {
+                if ( $args{Short} ) {
+                    $locstr = '> [_1]W';           # loc
+                }
+                else {
+                    $locstr = '> [_1] weeks';      # loc
+                }
+                $unit = $coef * 4;
+            }
+        }
+        elsif ( $args{Unit} eq 'month' ) {
+            if ( $duration < $MONTH ) {
+                if ( $args{Short} ) {
+                    $locstr = '< [_1]M';           # loc
+                }
+                else {
+                    $locstr = '< [_1] month';      # loc
+                }
+                $unit = 1;
+            }
+            elsif ( $duration > $coef * 12 * $MONTH ) {
+                if ( $args{Short} ) {
+                    $locstr = '> [_1]M';           # loc
+                }
+                else {
+                    $locstr = '> [_1] months';     # loc
+                }
+                $unit = $coef * 12;
+            }
+        }
+        elsif ( $args{Unit} eq 'year' ) {
+            if ( $duration < $YEAR ) {
+                if ( $args{Short} ) {
+                    $locstr = '< [_1]Y';           # loc
+                }
+                else {
+                    $locstr = '< [_1] years';      # loc
+                }
+                $unit = 1;
+            }
+            elsif ( $duration > $coef * 100 * $YEAR ) {
+                if ( $args{Short} ) {
+                    $locstr = '> [_1]Y';           # loc
+                }
+                else {
+                    $locstr = '> [_1] years';      # loc
+                }
+                $unit = $coef * 100;
+            }
+        }
+
+        if ($locstr) {
+            push @res, $self->loc($locstr, $unit);
+            $duration = 0;
+        }
+    }
+
     while ( $duration > 0 && ++$i <= $args{'Show'} ) {
 
         my ($locstr, $unit);
-        if ( $duration < $MINUTE ) {
+        if ( !$skip{second} && ( $duration < $MINUTE || $args{MaxUnit} =~ /second/i ) ) {
             $locstr = $args{Short}
                     ? '[_1]s'                      # loc
                     : '[quant,_1,second,seconds]'; # loc
             $unit = 1;
         }
-        elsif ( $duration < ( $coef * $HOUR ) ) {
+        elsif ( !$skip{minute} && ( $duration < ( $coef * $HOUR ) || $args{MaxUnit} =~ /minute/i ) ) {
             $locstr = $args{Short}
                     ? '[_1]m'                      # loc
                     : '[quant,_1,minute,minutes]'; # loc
             $unit = $MINUTE;
         }
-        elsif ( $duration < ( $coef * $DAY ) ) {
+        elsif ( !$skip{hour} && ( $duration < ( $coef * $DAY ) || $args{MaxUnit} =~ /hour/i) ) {
             $locstr = $args{Short}
                     ? '[_1]h'                      # loc
                     : '[quant,_1,hour,hours]';     # loc
             $unit = $HOUR;
         }
-        elsif ( $duration < ( $coef * $WEEK ) ) {
+        elsif ( !$skip{day} && ( $duration < ( $coef * $WEEK ) || $args{MaxUnit} =~ /day/i ) ) {
             $locstr = $args{Short}
                     ? '[_1]d'                      # loc
                     : '[quant,_1,day,days]';       # loc
             $unit = $DAY;
         }
-        elsif ( $duration < ( $coef * $MONTH ) ) {
+        elsif ( !$skip{week} && ( $duration < ( $coef * $MONTH ) || $args{MaxUnit} =~ /week/i) ) {
             $locstr = $args{Short}
                     ? '[_1]W'                      # loc
                     : '[quant,_1,week,weeks]';     # loc
             $unit = $WEEK;
         }
-        elsif ( $duration < $YEAR ) {
+        elsif ( !$skip{month} && ( $duration < $YEAR || $args{MaxUnit} =~ /month/i ) ) {
             $locstr = $args{Short}
                     ? '[_1]M'                      # loc
                     : '[quant,_1,month,months]';   # loc
@@ -762,6 +974,8 @@ sub LocalizedDateTime
                  @_,
                );
 
+    my $dt = $self->DateTimeObj(%args);
+
     # Require valid names for the format methods
     my $date_format = $args{DateFormat} =~ /^\w+$/
                     ? $args{DateFormat} : 'date_format_full';
@@ -775,24 +989,6 @@ sub LocalizedDateTime
     $date_format =~ s/EEEE/EEE/g if ( $args{'AbbrDay'} );
     $date_format =~ s/MMMM/MMM/g if ( $args{'AbbrMonth'} );
 
-    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydaym,$isdst,$offset) =
-                            $self->Localtime($args{'Timezone'});
-    $mon++;
-    my $tz = $self->Timezone($args{'Timezone'});
-
-    # FIXME : another way to call this module without conflict with local
-    # DateTime method?
-    my $dt = DateTime::->new( locale => $formatter,
-                            time_zone => $tz,
-                            year => $year,
-                            month => $mon,
-                            day => $mday,
-                            hour => $hour,
-                            minute => $min,
-                            second => $sec,
-                            nanosecond => 0,
-                          );
-
     if ( $args{'Date'} && !$args{'Time'} ) {
         return $dt->format_cldr($date_format);
     } elsif ( !$args{'Date'} && $args{'Time'} ) {
@@ -1163,6 +1359,58 @@ sub IsSet {
 
 }
 
+=head3 DateTimeObj [Timezone => 'utc']
+
+Returns an L<DateTime> object representing the same time as this RT::Date. The
+DateTime object's locale is set up to match the user's language.
+
+Modifying this DateTime object will not change the corresponding RT::Date, and
+vice versa.
+
+=cut
+
+sub DateTimeObj {
+    my $self = shift;
+    my %args = (
+        Timezone => '',
+        @_,
+    );
+
+    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$ydaym,$isdst,$offset) =
+                            $self->Localtime($args{'Timezone'});
+    $mon++;
+
+    return DateTime::->new(
+        locale     => $self->LocaleObj,
+        time_zone  => $self->Timezone($args{'Timezone'}),
+        year       => $year,
+        month      => $mon,
+        day        => $mday,
+        hour       => $hour,
+        minute     => $min,
+        second     => $sec,
+        nanosecond => 0,
+    );
+}
+
+=head3 Strftime FORMAT, [Timezone => 'user']
+
+Stringify the RT::Date according to the specified format. See
+L<DateTime/strftime Patterns>.
+
+=cut
+
+sub Strftime {
+    my $self = shift;
+    my $format = shift;
+    my %args = (
+        Timezone => 'user',
+        @_,
+    );
+
+    my $dt = $self->DateTimeObj(%args);
+    return $dt->strftime($format);
+}
 
 RT::Base->_ImportOverlays();
 
