@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -104,16 +104,19 @@ sub gv_escape($) {
     return $value;
 }
 
+sub loc { return HTML::Mason::Commands::loc(@_) };
+
 our (%fill_cache, @available_colors) = ();
 
 our %property_cb = (
-    Queue => sub { return $_[0]->QueueObj->Name || $_[0]->Queue },
-    CF    => sub {
+    Queue  => sub { return $_[0]->QueueObj->Name || $_[0]->Queue },
+    Status => sub { return loc($_[0]->Status) },
+    CF     => sub {
         my $values = $_[0]->CustomFieldValues( $_[1] );
         return join ', ', map $_->Content, @{ $values->ItemsArrayRef };
     },
 );
-foreach my $field (qw(Subject Status TimeLeft TimeWorked TimeEstimated)) {
+foreach my $field (qw(Subject TimeLeft TimeWorked TimeEstimated)) {
     $property_cb{ $field } = sub { return $_[0]->$field },
 }
 foreach my $field (qw(Creator LastUpdatedBy Owner)) {
@@ -136,12 +139,33 @@ foreach my $field (qw(Told Starts Started Due Resolved LastUpdated Created)) {
 }
 foreach my $field (qw(Members DependedOnBy ReferredToBy)) {
     $property_cb{ $field } = sub {
-        return join ', ', map $_->BaseObj->id, @{ $_[0]->$field->ItemsArrayRef };
+        my $links = $_[0]->$field;
+        my @string;
+        while ( my $link = $links->Next ) {
+            next if UNIVERSAL::isa($link->BaseObj, 'RT::Article') && $link->BaseObj->Disabled;
+            next if $field eq 'ReferredToBy' && UNIVERSAL::isa($link->BaseObj, 'RT::Ticket') && $link->BaseObj->Type eq 'reminder';
+            my $prefix =
+                UNIVERSAL::isa( $link->BaseObj, 'RT::Ticket' ) ? ''
+              : UNIVERSAL::isa( $link->BaseObj, 'RT::Article' ) ? 'a:'
+              :                                                   $link->BaseURI->Scheme . ':';
+            push @string, $prefix . $link->BaseObj->id;
+        }
+        return join ', ', @string;
     };
 }
 foreach my $field (qw(MemberOf DependsOn RefersTo)) {
     $property_cb{ $field } = sub {
-        return join ', ', map $_->TargetObj->id, @{ $_[0]->$field->ItemsArrayRef };
+        my $links = $_[0]->$field;
+        my @string;
+        while ( my $link = $links->Next ) {
+            next if UNIVERSAL::isa($link->TargetObj, 'RT::Article') && $link->TargetObj->Disabled;
+            my $prefix =
+                UNIVERSAL::isa( $link->TargetObj, 'RT::Ticket' ) ? ''
+              : UNIVERSAL::isa( $link->TargetObj, 'RT::Article' ) ? 'a:'
+              :                                                   $link->TargetURI->Scheme . ':';
+            push @string, $prefix . $link->TargetObj->id;
+        }
+        return join ', ', @string;
     };
 }
 
@@ -163,7 +187,8 @@ sub TicketProperties {
         next if $seen{ lc $cf->Name }++;
         next if $cf->Type eq 'Image';
         if ( $first ) {
-            push @res, 'CustomFields', [];
+            push @res, 'Custom Fields', # loc
+                [];
             $first = 0;
         }
         push @{ $res[-1] }, 'CF.{'. $cf->Name .'}';
@@ -203,7 +228,8 @@ sub _PropertiesToFields {
             $RT::Logger->error("Couldn't find property handler for '$key' and '@subkeys' subkeys");
             next;
         }
-        push @fields, ($subkeys[0] || $key) .': '. $property_cb{ $key }->( $args{'Ticket'}, @subkeys );
+        my $label = $key eq 'CF' ? $subkeys[0] : loc($key);
+        push @fields, $label .': '. $property_cb{ $key }->( $args{'Ticket'}, @subkeys );
     }
 
     return @fields;
@@ -299,9 +325,12 @@ sub TicketLinks {
     }
 
     $args{'Seen'} ||= {};
-    return $args{'Graph'} if $args{'Seen'}{ $args{'Ticket'}->id }++;
-
-    $self->AddTicket( %args );
+    if ( $args{'Seen'}{ $args{'Ticket'}->id } && $args{'Seen'}{ $args{'Ticket'}->id } <= $args{'CurrentDepth'} ) {
+      return $args{'Graph'};
+    } elsif ( ! defined $args{'Seen'}{ $args{'Ticket'}->id } ) {
+      $self->AddTicket( %args );
+    }
+    $args{'Seen'}{ $args{'Ticket'}->id } = $args{'CurrentDepth'};
 
     return $args{'Graph'} if $args{'MaxDepth'} && $args{'CurrentDepth'} >= $args{'MaxDepth'};
 
