@@ -274,7 +274,7 @@ sub __LinearizeTree {
 
             if ( $op =~ /^IS( NOT)?$/i ) {
                 $value = 'NULL';
-            } elsif ( $value !~ /^[+-]?[0-9]+$/ ) {
+            } elsif ( $clause->{QuoteValue} ) {
                 $value =~ s/(['\\])/\\$1/g;
                 $value = "'$value'";
             }
@@ -324,7 +324,37 @@ sub ParseSQL {
     $callback{'CloseParen'} = sub { $node = $node->getParent };
     $callback{'EntryAggregator'} = sub { $node->setNodeValue( $_[0] ) };
     $callback{'Condition'} = sub {
-        my ($key, $op, $value) = @_;
+        my ($key, $op, $value, $value_is_quoted) = @_;
+
+        if (  !$value_is_quoted
+            && $key   !~ /(?:CustomField|CF)\./
+            && $value =~ /(?:CustomField|CF)\./
+            && RT->Config->Get('DatabaseType') eq 'Pg' )
+        {
+
+            # E.g. LastUpdated > CF.{Beta Date}
+            #
+            # Pg 9 tries to cast all ObjectCustomFieldValues to datetime,
+            # which could fail since not all custom fields are of DateTime
+            # type. To get around this issue, here we switch the key/value
+            # pair to compare as text instead.
+
+            my ($major_version) = $RT::Handle->dbh->selectrow_array("SHOW server_version") =~ /^(\d+)/;
+            if ( $major_version < 10 ) {
+                my %reverse = (
+                    '>'  => '<',
+                    '>=' => '<=',
+                    '<'  => '>',
+                    '<=' => '>=',
+                    '='  => '=',
+                );
+                if ( $reverse{$op} ) {
+                    RT->Logger->debug("Switching $key/$value to compare using text");
+                    ( $key, $value ) = ( $value, $key );
+                    $op = $reverse{$op};
+                }
+            }
+        }
 
         my ($main_key, $subkey) = split /[.]/, $key, 2;
 
@@ -333,12 +363,21 @@ sub ParseSQL {
         }
         $main_key = $lcfield{ lc $main_key };
 
+        if ( $op =~ /^SHALLOW\s+/i && ($main_key !~ /(?:Requestor|Owner|AdminCc|Cc|CustomRole)/) ) {
+            push @results, [ $args{'CurrentUser'}->loc("Unsupported operator: [_1]", $op), -1 ];
+        }
+
         # Hardcode value for IS / IS NOT
         $value = 'NULL' if $op =~ /^IS( NOT)?$/i;
 
-        my $clause = { Key => $main_key, Subkey => $subkey,
-                       Meta => $field{ $main_key },
-                       Op => $op, Value => $value };
+        my $clause = {
+            Key           => $main_key,
+            Subkey        => $subkey,
+            Meta          => $field{$main_key},
+            Op            => $op,
+            Value         => $value,
+            QuoteValue    => $value_is_quoted,
+        };
         $node->addChild( __PACKAGE__->new( $clause ) );
     };
     $callback{'Error'} = sub { push @results, @_ };
