@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -127,9 +127,8 @@ sub Delete {
     if ($rv) {
         return ($rv, $self->loc("Object deleted"));
     } else {
-
-        return(0, $self->loc("Object could not be deleted"))
-    } 
+        return (0, $self->loc("Object could not be deleted"));
+    }
 }
 
 =head2 RecordType
@@ -145,28 +144,6 @@ sub RecordType {
     return $res;
 }
 
-=head2 ObjectTypeStr
-
-DEPRECATED. Stays here for backwards. Returns localized L</RecordType>.
-
-=cut
-
-# we deprecate because of:
-# * ObjectType is used in several classes with ObjectId to store
-#   records of different types, for example transactions use those
-#   and it's unclear what this method should return 'Transaction'
-#   or type of referenced record
-# * returning localized thing is not good idea
-
-sub ObjectTypeStr {
-    my $self = shift;
-    RT->Deprecated(
-        Remove => "4.4",
-        Instead => "RecordType",
-    );
-    return $self->loc( $self->RecordType( @_ ) );
-}
-
 =head2 Attributes
 
 Return this object's attributes as an RT::Attributes object
@@ -394,6 +371,7 @@ sub LoadByCols {
 
     # We don't want to hang onto this
     $self->ClearAttributes;
+    delete $self->{_Roles};
 
     unless ( $self->_Handle->CaseSensitive ) {
         my ( $ret, $msg ) = $self->SUPER::LoadByCols( @_ );
@@ -445,30 +423,6 @@ sub CreatedObj {
 }
 
 
-# B<DEPRECATED> and will be removed in 4.4
-sub AgeAsString {
-    my $self = shift;
-    RT->Deprecated(
-        Remove => "4.4",
-        Instead => "->CreatedObj->AgeAsString",
-    );
-    return ( $self->CreatedObj->AgeAsString() );
-}
-
-# B<DEPRECATED> and will be removed in 4.4
-sub LongSinceUpdateAsString {
-    my $self = shift;
-    RT->Deprecated(
-        Remove => "4.4",
-        Instead => "->LastUpdatedObj->AgeAsString",
-    );
-    if ( $self->LastUpdated ) {
-        return ( $self->LastUpdatedObj->AgeAsString() );
-    } else {
-        return "never";
-    }
-}
-
 sub LastUpdatedAsString {
     my $self = shift;
     if ( $self->LastUpdated ) {
@@ -857,28 +811,31 @@ sub _EncodeLOB {
 
 =head2 _DecodeLOB C<ContentType>, C<ContentEncoding>, C<Content>
 
-Unpacks data stored in the database, which may be base64 or QP encoded
-because of our need to store binary and badly encoded data in columns
-marked as UTF-8.  Databases such as PostgreSQL and Oracle care that you
-are feeding them invalid UTF-8 and will refuse the content.  This
-function handles unpacking the encoded data.
+This function reverses the effects of L</_EncodeLOB>, by unpacking the
+data, provided as bytes (not characters!), from the database.  This
+data may also be Base64 or Quoted-Printable encoded, as given by
+C<Content-Encoding>.  This encoding layer exists because the
+underlying database column is "text", which rejects non-UTF-8 byte
+sequences.
 
-It returns textual data as a UTF-8 string which has been processed by Encode's
-PERLQQ filter which will replace the invalid bytes with \x{HH} so you can see
-the invalid byte but won't run into problems treating the data as UTF-8 later.
+Alternatively, if the data lives in external storage, it will be read
+(or downloaded) and returned.
 
-This is similar to how we filter all data coming in via the web UI in
-RT::Interface::Web::DecodeARGS. This filter should only end up being
-applied to old data from less UTF-8-safe versions of RT.
+This function differs in one important way from being the inverse of
+L</_EncodeLOB>: for textual data (as judged via
+L<RT::I18N/IsTextualContentType> applied to the given C<ContentType>),
+C<_DecodeLOB> returns character strings, not bytes.  The character set
+used in decoding is taken from the C<ContentType>, or UTF-8 if not
+provided; however, for all textual content inserted by current code,
+the character set used for storage is always UTF-8.
 
-If the passed C<ContentType> includes a character set, that will be used
-to decode textual data; the default character set is UTF-8.  This is
-necessary because while we attempt to store textual data as UTF-8, the
-definition of "textual" has migrated over time, and thus we may now need
-to attempt to decode data that was previously not trancoded on insertion.
-
-Important Note - This function expects an octet string and returns a
-character string for non-binary data.
+This decoding step is done using L<Encode>'s PERLQQ filter, which
+replaces invalid byte sequences with C<\x{HH}>.  This mirrors how data
+from query parameters are parsed in L<RT::Interface::Web/DecodeARGS>.
+Since RT is now strict about the bytes it inserts, substitution
+characters should only be needed for data inserted by older versions
+of RT, or for C<ContentType>s which are now believed to be textual,
+but were not considered so on insertion (and thus not transcoded).
 
 =cut
 
@@ -896,9 +853,24 @@ sub _DecodeLOB {
     elsif ( $ContentEncoding eq 'quoted-printable' ) {
         $Content = MIME::QuotedPrint::decode($Content);
     }
+    elsif ( $ContentEncoding eq 'external' ) {
+        my $Digest = $Content;
+        my $Storage = RT->System->ExternalStorage;
+        unless ($Storage) {
+            RT->Logger->error( "Failed to load $Content; external storage not configured" );
+            return ("");
+        };
+
+        ($Content, my $msg) = $Storage->Get( $Digest );
+        unless (defined $Content) {
+            RT->Logger->error( "Failed to load $Digest from external storage: $msg" );
+            return ("");
+        }
+    }
     elsif ( $ContentEncoding && $ContentEncoding ne 'none' ) {
         return ( $self->loc( "Unknown ContentEncoding [_1]", $ContentEncoding ) );
     }
+
     if ( RT::I18N::IsTextualContentType($ContentType) ) {
         my $entity = MIME::Entity->new();
         $entity->head->add("Content-Type", $ContentType);
@@ -971,13 +943,16 @@ sub Update {
         # items. If it fails, we don't care
         do {
             no warnings "uninitialized";
-            local $@;
-            my $name = eval {
-                my $object = $attribute . "Obj";
-                $self->$object->Name;
-            };
-            unless ($@) {
-                next if $name eq $value || $name eq ($value || 0);
+
+            if ( $attribute ne 'Lifecycle' ) {
+                local $@;
+                my $name = eval {
+                    my $object = $attribute . "Obj";
+                    $self->$object->Name;
+                };
+                unless ($@) {
+                    next if $name eq $value || $name eq ( $value || 0 );
+                }
             }
 
             next if $truncated_value eq $self->$attribute();
@@ -1668,7 +1643,6 @@ sub _NewTransaction {
         Field     => undef,
         MIMEObj   => undef,
         ActivateScrips => 1,
-        CommitScrips => 1,
         SquelchMailTo => undef,
         @_
     );
@@ -1707,8 +1681,8 @@ sub _NewTransaction {
         ReferenceType => $ref_type,
         MIMEObj   => $args{'MIMEObj'},
         ActivateScrips => $args{'ActivateScrips'},
-        CommitScrips => $args{'CommitScrips'},
-        SquelchMailTo => $args{'SquelchMailTo'},
+        DryRun => $self->{DryRun},
+        SquelchMailTo => $args{'SquelchMailTo'} || $self->{TransSquelchMailTo},
     );
 
     # Rationalize the object since we may have done things to it during the caching.
@@ -1722,7 +1696,7 @@ sub _NewTransaction {
         $self->_UpdateTimeTaken( $args{'TimeTaken'}, Transaction => $trans );
     }
     if ( RT->Config->Get('UseTransactionBatch') and $transaction ) {
-            push @{$self->{_TransactionBatch}}, $trans if $args{'CommitScrips'};
+        push @{$self->{_TransactionBatch}}, $trans;
     }
 
     RT->DatabaseHandle->Commit unless $in_txn;
@@ -1759,13 +1733,24 @@ sub Transactions {
 Returns the result of L</Transactions> ordered per the
 I<OldestTransactionsFirst> preference/option.
 
+Pass an optional value 'ASC' or 'DESC' to force a specific
+order.
+
 =cut
 
 sub SortedTransactions {
     my $self  = shift;
+    my $order = shift || 0;
     my $txns  = $self->Transactions;
-    my $order = RT->Config->Get("OldestTransactionsFirst", $self->CurrentUser)
-        ? 'ASC' : 'DESC';
+
+    if ( $order && ( $order eq 'ASC' || $order eq 'DESC' ) ) {
+        # Use provided value
+    }
+    else {
+        $order = RT->Config->Get("OldestTransactionsFirst", $self->CurrentUser)
+            ? 'ASC' : 'DESC';
+    }
+
     $txns->OrderByCols(
         { FIELD => 'Created',   ORDER => $order },
         { FIELD => 'id',        ORDER => $order },
@@ -1800,6 +1785,7 @@ our %TRANSACTION_CLASSIFICATION = (
             Owner Creator LastUpdatedBy
         ) ),
     },
+    CustomField => 'cfs',
     SystemError => 'error',
     AttachmentTruncate => 'attachment-truncate',
     AttachmentDrop => 'attachment-drop',
@@ -1963,10 +1949,13 @@ sub _AddCustomFieldValue {
         LargeContent      => undef,
         ContentType       => undef,
         RecordTransaction => 1,
+        ForCreation       => 0,
         @_
     );
 
     my $cf = $self->LoadCustomFieldByIdentifier($args{'Field'});
+    $cf->{include_set_initial} = 1 if $args{'ForCreation'};
+
     unless ( $cf->Id ) {
         return ( 0, $self->loc( "Custom field [_1] not found", $args{'Field'} ) );
     }
@@ -2041,6 +2030,7 @@ sub _AddCustomFieldValue {
             Content      => $args{'Value'},
             LargeContent => $args{'LargeContent'},
             ContentType  => $args{'ContentType'},
+            ForCreation  => $args{'ForCreation'},
         );
 
         unless ( $new_value_id ) {
@@ -2048,14 +2038,9 @@ sub _AddCustomFieldValue {
         }
 
         my $new_value = RT::ObjectCustomFieldValue->new( $self->CurrentUser );
+        $new_value->{include_set_initial} = 1 if $args{'ForCreation'};
         $new_value->Load( $new_value_id );
 
-        # now that adding the new value was successful, delete the old one
-        if ( $old_value ) {
-            my ( $val, $msg ) = $old_value->Delete();
-            return ( 0, $msg ) unless $val;
-        }
-
         if ( $args{'RecordTransaction'} ) {
             my ( $TransactionId, $Msg, $TransactionObj ) =
               $self->_NewTransaction(
@@ -2112,11 +2097,13 @@ sub _AddCustomFieldValue {
             Content      => $args{'Value'},
             LargeContent => $args{'LargeContent'},
             ContentType  => $args{'ContentType'},
+            ForCreation  => $args{'ForCreation'},
         );
 
         unless ( $new_value_id ) {
             return ( 0, $self->loc( "Could not add new custom field value: [_1]", $msg ) );
         }
+
         if ( $args{'RecordTransaction'} ) {
             my ( $tid, $msg ) = $self->_NewTransaction(
                 Type          => 'CustomField',
@@ -2132,7 +2119,80 @@ sub _AddCustomFieldValue {
     }
 }
 
+=head2 AddCustomFieldDefaultValues
 
+Add default values to object's empty custom fields.
+
+=cut
+
+sub AddCustomFieldDefaultValues {
+    my $self = shift;
+
+    my $object = ( ref $self )->new( RT->SystemUser );
+    $object->Load( $self->id );
+
+    my $cfs  = $object->CustomFields;
+    my @msgs;
+    while ( my $cf = $cfs->Next ) {
+        next if $object->CustomFieldValues($cf->id)->Count || !$cf->SupportDefaultValues;
+        my ( $on ) = grep { $_->isa( $cf->RecordClassFromLookupType ) } $cf->ACLEquivalenceObjects;
+        my $values = $cf->DefaultValues( Object => $on || RT->System );
+        foreach my $value ( UNIVERSAL::isa( $values => 'ARRAY' ) ? @$values : $values ) {
+            next if $object->CustomFieldValueIsEmpty(
+                Field => $cf,
+                Value => $value,
+            );
+
+            my ( $status, $msg ) = $object->_AddCustomFieldValue(
+                Field             => $cf->id,
+                Value             => $value,
+                RecordTransaction => 0,
+            );
+            push @msgs, $msg unless $status;
+        }
+    }
+    return ( 0, @msgs ) if @msgs;
+    return 1;
+}
+
+=head2 CustomFieldValueIsEmpty { Field => FIELD, Value => VALUE }
+
+Check if the custom field value is empty.
+
+Some custom fields could have other special empty values, e.g. "1970-01-01" is empty for Date cf
+
+Return 1 if it is empty, 0 otherwise.
+
+=cut
+
+
+sub CustomFieldValueIsEmpty {
+    my $self = shift;
+    my %args = (
+        Field => undef,
+        Value => undef,
+        @_
+    );
+    my $value = $args{Value};
+    return 1 unless defined $value  && length $value;
+
+    my $cf = ref($args{'Field'})
+           ? $args{'Field'}
+           : $self->LoadCustomFieldByIdentifier( $args{'Field'} );
+
+    if ($cf) {
+        if ( $cf->__Value('Type') =~ /^Date(?:Time)?$/ ) {
+            my $DateObj = RT::Date->new( $self->CurrentUser );
+            $DateObj->Set(
+                Format => 'unknown',
+                Value  => $value,
+                $cf->Type eq 'Date' ? ( Timezone => 'UTC' ) : (),
+            );
+            return 1 unless $DateObj->IsSet;
+        }
+    }
+    return 0;
+}
 
 =head2 DeleteCustomFieldValue { Field => FIELD, Value => VALUE }
 
@@ -2360,6 +2420,315 @@ sub WikiBase {
     return RT->Config->Get('WebPath'). "/index.html?q=";
 }
 
+# Matches one field in "field - field" style range specs. Subclasses
+# that can participate in custom date ranges should override this method
+# to match their additional date fields. Be sure to call this superclass
+# method to get "now", datetime columns and CF parsing.
+
+sub _CustomDateRangeFieldParser {
+    my $self = shift;
+    my $regex = qr{
+        now
+        | cf\. (?: \{ .*? \} | \S+ )
+    }xi;
+
+    for my $column ( keys %{ $_TABLE_ATTR->{ ref $self || $self} } ) {
+        my $entry = $_TABLE_ATTR->{ ref $self || $self}{$column};
+        next unless $entry->{read} && ( $entry->{type} // '' ) eq 'datetime';
+        $regex .= '|' . qr{$column}i;
+    }
+    return $regex;
+}
+
+# Returns an RT::Date instantiated with this record's value for the parsed
+# field name. Includes the $range_name parameter only for diagnostics.
+# Subclasses should override this to instantiate the fields they added in
+# _CustomDateRangeFieldParser.
+
+sub _DateForCustomDateRangeField {
+    my $self       = shift;
+    my $field      = shift;
+    my $range_name = shift;
+
+    my $date = RT::Date->new($self->CurrentUser);
+
+    if (lc($field) eq 'now') {
+        $date->Set(Format => 'unix', Value => time);
+    }
+    elsif ($field =~ m{^ cf\. (?: \{ (.*?) \} | (\S+) ) $}xi) {
+        my $name = $1 || $2;
+        my $value = $self->FirstCustomFieldValue($name);
+
+        if (!$value) {
+            # no CF value for this record, so bail out
+            return;
+        }
+
+        $date->Set(Format => 'unknown', Value => $value, Timezone => 'UTC');
+    }
+    else {
+        if ( my ($column) = grep { lc $field eq lc $_ } keys %{ $_TABLE_ATTR->{ ref $self || $self } } ) {
+            my $method = $column . 'Obj';
+            if ( $self->can($method) ) {
+                $date = $self->$method;
+            }
+            else {
+                RT->Logger->error( "Missing $method in " . ref $self );
+                return;
+            }
+        }
+        else {
+            RT->Logger->error("Unable to parse '$field' as a field name in CustomDateRanges '$range_name'");
+            return;
+        }
+    }
+
+    return $date;
+}
+
+# Parses custom date range spec and returns a hash containing parsed info.
+# Returns the empty list if there's an error.
+
+sub _ParseCustomDateRangeSpec {
+    my $self = shift;
+    my $name = shift;
+    my $spec = shift;
+
+    my $calculation;
+    my $format;
+
+    if (ref($spec)) {
+        $calculation = $spec->{value} || join( ' - ', $spec->{to}, $spec->{from} );
+        $format = $spec->{format};
+    }
+    else {
+        $calculation = $spec;
+    }
+
+    if (!$calculation || ref($calculation)) {
+        RT->Logger->error("CustomDateRanges '$name' 'value' must be a string");
+        return;
+    }
+
+    if ($format && ref($format) ne 'CODE') {
+        RT->Logger->error("CustomDateRanges '$name' 'format' must be a CODE reference");
+        return;
+    }
+
+    # class-specific matcher for now, created, CF.{foo bar}, CF.baz, etc.
+    my $field_parser = $self->_CustomDateRangeFieldParser;
+
+    # regex parses "field - field" (intentionally very strict)
+    my $calculation_parser = qr{
+        ^
+        ($field_parser)   # to field name
+        \s+ - \s+       # space, operator, more space
+        ($field_parser)   # from field name
+        $
+    }x;
+
+    my @matches = $calculation =~ $calculation_parser;
+
+    if (!@matches) {
+        RT->Logger->error("Unable to parse '$calculation' as a calculated value in CustomDateRanges '$name'");
+        return;
+    }
+
+    if ( ref $spec ) {
+        for my $type ( qw/from to/ ) {
+            if ( $spec->{"${type}_fallback"} && $spec->{"${type}_fallback"} !~ /^$field_parser$/ ) {
+                RT->Logger->error( "Invalid ${type}_fallback field: " . $spec->{"${type}_fallback"} );
+                return;
+            }
+        }
+    }
+
+    my %date_range_spec = ( from => $matches[1], to => $matches[0], ref $spec ? %$spec : () );
+    return %date_range_spec;
+}
+
+=head2 CustomDateRange name, spec
+
+Takes a L<RT_Config/%CustomDateRanges>-style spec string and its name (for
+diagnostics). Returns a localized string evaluating the calculation. If either
+date is unset, or anything fails to parse, this returns C<undef>.
+
+=cut
+
+sub CustomDateRange {
+    my $self = shift;
+    my $name = shift;
+    my $spec = shift;
+
+    my %date_range_spec = $self->_ParseCustomDateRangeSpec($name, $spec);
+
+    # parse failed; render no value
+    return unless $date_range_spec{from} && $date_range_spec{to};
+
+    my $end_dt = $self->_DateForCustomDateRangeField($date_range_spec{to}, $name);
+    my $start_dt = $self->_DateForCustomDateRangeField($date_range_spec{from}, $name);
+
+    unless ( $start_dt && $start_dt->IsSet ) {
+        if ( ref $spec && $date_range_spec{from_fallback} ) {
+            $start_dt = $self->_DateForCustomDateRangeField( $date_range_spec{from_fallback}, $name );
+        }
+    }
+
+    unless ( $end_dt && $end_dt->IsSet ) {
+        if ( ref $spec && $date_range_spec{to_fallback} ) {
+            $end_dt = $self->_DateForCustomDateRangeField( $date_range_spec{to_fallback}, $name );
+        }
+    }
+
+    # RT::Date instantiation failed; render no value
+    return unless $start_dt && $start_dt->IsSet
+               && $end_dt && $end_dt->IsSet;
+
+    my $duration;
+    if ( $date_range_spec{business_time} ) {
+        my $schedule;
+        my $timezone;
+
+        # Prefer the schedule/timezone specified in %ServiceAgreements for current object
+        if ( $self->isa('RT::Ticket') && !$self->QueueObj->SLADisabled && $self->SLA ) {
+            if ( my $config = RT->Config->Get('ServiceAgreements') ) {
+                if ( ref( $config->{QueueDefault}{ $self->QueueObj->Name } ) eq 'HASH' ) {
+                    $timezone = $config->{QueueDefault}{ $self->QueueObj->Name }{Timezone};
+                }
+
+                # Each SLA could have its own schedule and timezone
+                if ( my $agreement = $config->{Levels}{ $self->SLA } ) {
+                    $schedule = $agreement->{BusinessHours};
+                    $timezone ||= $agreement->{Timezone};
+                }
+            }
+        }
+        $timezone ||= RT->Config->Get('Timezone');
+        $schedule ||= 'Default';
+
+        {
+            local $ENV{'TZ'} = $ENV{'TZ'};
+            if ( $timezone ne ( $ENV{'TZ'} || '' ) ) {
+                $ENV{'TZ'} = $timezone;
+                require POSIX;
+                POSIX::tzset();
+            }
+
+            my $bhours = RT::SLA->BusinessHours($schedule);
+            $duration = $bhours->between(
+                $start_dt->Unix <= $end_dt->Unix
+                ? ( $start_dt->Unix, $end_dt->Unix )
+                : ( $end_dt->Unix, $start_dt->Unix )
+            );
+            $duration *= -1 if $start_dt->Unix > $end_dt->Unix;
+        }
+
+        if ( $timezone ne ( $ENV{'TZ'} || '' ) ) {
+            POSIX::tzset();
+        }
+    }
+
+    $duration //= $end_dt->Diff($start_dt);
+
+    # _ParseCustomDateRangeSpec guarantees $format is a coderef
+    if ($date_range_spec{format}) {
+        return $date_range_spec{format}->($duration, $end_dt, $start_dt, $self);
+    }
+    else {
+        my $max_unit = $date_range_spec{business_time} ? 'hour' : 'year';
+
+        # "x days ago" is strongly suggestive of comparing with the current
+        # time; but if we're comparing two arbitrary times, "x days prior"
+        # reads better
+        if ($duration < 0) {
+            $duration *= -1;
+            return $self->loc('[_1] prior', $end_dt->DurationAsString($duration, MaxUnit => $max_unit));
+        }
+        else {
+            return $end_dt->DurationAsString($duration, MaxUnit => $max_unit);
+        }
+    }
+}
+
+=head2 CustomDateRanges
+
+Return all of the custom date ranges of current class.
+
+=cut
+
+sub CustomDateRanges {
+    my $self = shift;
+    my %args = (
+        Type          => undef,
+        ExcludeSystem => undef,
+        ExcludeUsers  => undef,
+        ExcludeUser   => undef,
+        @_,
+    );
+
+    my $type = $args{Type} || ref $self || $self,;
+    my %ranges;
+
+    if ( !$args{ExcludeSystem} ) {
+        if ( my $config = RT->Config->Get('CustomDateRanges') ) {
+            for my $name ( keys %{ $config->{$type} || {} } ) {
+                $ranges{$name} ||= $config->{$type}{$name};
+            }
+        }
+
+        if ( my $db_config = RT->Config->Get('CustomDateRangesUI') ) {
+            for my $name ( keys %{ $db_config->{$type} || {} } ) {
+                $ranges{$name} ||= $db_config->{$type}{$name};
+            }
+        }
+    }
+
+    if ( !$args{ExcludeUsers} ) {
+        my $attributes = RT::Attributes->new( RT->SystemUser );
+        $attributes->Limit( FIELD => 'Name',       VALUE => 'Pref-CustomDateRanges' );
+        $attributes->Limit( FIELD => 'ObjectType', VALUE => 'RT::User' );
+        if ( $args{ExcludeUser} ) {
+            $attributes->Limit( FIELD => 'Creator', OPERATOR => '!=', VALUE => $args{ExcludeUser} );
+        }
+        $attributes->OrderBy( FIELD => 'id' );
+
+        while ( my $attribute = $attributes->Next ) {
+            if ( my $content = $attribute->Content ) {
+                for my $name ( keys %{ $content->{$type} || {} } ) {
+                    $ranges{$name} ||= $content->{$type}{$name};
+                }
+            }
+        }
+    }
+    return %ranges;
+}
+
+=head2 CustomDateRangeFields
+
+Return all of the fields custom date range could use for current class.
+
+=cut
+
+sub CustomDateRangeFields {
+    my $self = shift;
+    my $type = ref $self || $self;
+
+    my @fields = 'now';
+
+    for my $column ( keys %{ $_TABLE_ATTR->{ ref $self || $self } } ) {
+        my $entry = $_TABLE_ATTR->{ ref $self || $self }{$column};
+        next unless $entry->{read} && ( $entry->{type} // '' ) eq 'datetime';
+        push @fields, $column;
+    }
+
+    my $cfs = RT::CustomFields->new( ref $self ? $self->CurrentUser : RT->SystemUser );
+    $cfs->Limit( FIELD => 'Type', VALUE => [ 'Date', 'DateTime' ], OPERATOR => 'IN' );
+    while ( my $cf = $cfs->Next ) {
+        push @fields, 'CF.{' . $cf->Name . '}';
+    }
+    return sort { lc $a cmp lc $b } @fields;
+}
+
 sub UID {
     my $self = shift;
     return undef unless defined $self->Id;
@@ -2378,15 +2747,23 @@ sub FindDependencies {
         }
     }
 
+    my $objs;
+
     # Object attributes, we have to check on every object
-    my $objs = $self->Attributes;
-    $deps->Add( in => $objs );
+    # attributes of attributes are not supported yet though.
+    if ( !$self->isa('RT::Attribute') ) {
+        $objs = $self->Attributes;
+        $deps->Add( in => $objs );
+    }
 
     # Transactions
     if (   $self->isa("RT::Ticket")
         or $self->isa("RT::User")
         or $self->isa("RT::Group")
         or $self->isa("RT::Article")
+        or $self->isa("RT::Asset")
+        or $self->isa("RT::Catalog")
+        or $self->isa("RT::Attribute")
         or $self->isa("RT::Queue") )
     {
         $objs = RT::Transactions->new( $self->CurrentUser );
@@ -2400,12 +2777,13 @@ sub FindDependencies {
          or $self->isa("RT::Ticket")
          or $self->isa("RT::User")
          or $self->isa("RT::Group")
+         or $self->isa("RT::Asset")
          or $self->isa("RT::Queue")
          or $self->isa("RT::Article") )
             and $self->can("CustomFieldValues") )
     {
         $objs = $self->CustomFieldValues; # Actually OCFVs
-        $objs->{find_expired_rows} = 1;
+        $objs->{find_disabled_rows} = 1;
         $deps->Add( in => $objs );
     }
 
@@ -2555,7 +2933,7 @@ sub _AsInsertQuery
 
     my $dbh = $RT::Handle->dbh;
 
-    my $res = "INSERT INTO ". $dbh->quote_identifier( $self->Table );
+    my $res = "INSERT INTO ". $self->Table;
     my $values = $self->{'values'};
     $res .= "(". join( ",", map { $dbh->quote_identifier( $_ ) } sort keys %$values ) .")";
     $res .= " VALUES";
@@ -2606,7 +2984,7 @@ sub __DependsOn
 
 # Object custom field values
     my $objs = $self->CustomFieldValues;
-    $objs->{'find_expired_rows'} = 1;
+    $objs->{'find_disabled_rows'} = 1;
     push( @$list, $objs );
 
 # Object attributes
@@ -2619,6 +2997,13 @@ sub __DependsOn
     $objs->Limit( FIELD => 'ObjectId', VALUE => $self->id );
     push( @$list, $objs );
 
+    if ( $self->isa( 'RT::CustomField' ) ) {
+        $objs = RT::Transactions->new( $self->CurrentUser );
+        $objs->Limit( FIELD => 'Type',  VALUE => 'CustomField' );
+        $objs->Limit( FIELD => 'Field', VALUE => $self->id );
+        push( @$list, $objs );
+    }
+
 # Links
     if ( $self->can('Links') ) {
         # make sure we don't skip any record
