@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2016 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -80,12 +80,13 @@ sub TestArgs
     my %args = @_;
 
     my @strings;
+    my $Organization = RT->Config->Get('Organization');
     foreach my $name( @RT::Shredder::SUPPORTED_OBJECTS ) {
         next unless $args{$name};
 
         my $list = $args{$name};
         $list = [$list] unless UNIVERSAL::isa( $list, 'ARRAY' );
-        push @strings, map "RT::$name\-$_", @$list;
+        push @strings, map "RT::$name-$Organization-$_", @$list;
     }
 
     my @objs = RT::Shredder->CastObjectsToRecords( Objects => \@strings );
