@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2019 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -49,7 +49,7 @@
 =head1 SYNOPSIS
 
   use RT::Squish::CSS;
-  my $squish = RT::Squish::CSS->new( Style => 'aileron');
+  my $squish = RT::Squish::CSS->new( Style => 'elevator-light');
 
 =head1 DESCRIPTION
 
