@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -64,6 +64,7 @@ use warnings;
 
 package RT::Squish::JS;
 use base 'RT::Squish';
+use JavaScript::Minifier::XS ();
 
 =head2 Squish
 
@@ -73,55 +74,32 @@ not only concatenate files, but also minify them
 
 sub Squish {
     my $self    = shift;
-    my $content;
+    my $content = "";
 
-    for my $file ( RT->Config->Get('JSFiles') ) {
-        my $path = "/NoAuth/js/$file";
-        if ( $HTML::Mason::Commands::m->comp_exists($path) ) {
-            $content .= $HTML::Mason::Commands::m->scomp($path);
+    for my $file ( RT::Interface::Web->JSFiles ) {
+        my $uri = $file =~ m{^/} ? $file : "/static/js/$file";
+        my $res = RT::Interface::Web::Handler->GetStatic($uri);
+
+        if ($res->is_success) {
+            if ( $file =~ /\.min\.js$/ ) {
+                $content .= $res->decoded_content . "\n";
+            }
+            else {
+                $content .= $self->Filter($res->decoded_content) . "\n";
+            }
         } else {
-            RT->Logger->error("Unable to open $path for JS Squishing");
+            RT->Logger->error("Unable to fetch $uri for JS Squishing: " . $res->status_line);
             next;
         }
     }
 
-    return $self->Filter($content);
-}
-
-sub Filter {
-    my $self    = shift;
-    my $content = shift;
-
-    my $minified;
-    my $jsmin = RT->Config->Get('JSMinPath');
-    if ( $jsmin && -x $jsmin ) {
-        my $input = $content;
-        my ( $output, $error );
-
-        # If we're running under fastcgi, STDOUT and STDERR are tied
-        # filehandles, which cause IPC::Run3 to flip out.  Construct
-        # temporary, not-tied replacements for it to see instead.
-        my $stdout = IO::Handle->new;
-        $stdout->fdopen( 1, 'w' );
-        local *STDOUT = $stdout;
-        my $stderr = IO::Handle->new;
-        $stderr->fdopen( 2, 'w' );
-        local *STDERR = $stderr;
-
-        local $SIG{'CHLD'} = 'DEFAULT';
-        require IPC::Run3;
-        IPC::Run3::run3( [$jsmin], \$input, \$output, \$error );
-        if ( $? >> 8 ) {
-            $RT::Logger->warning("failed to jsmin: $error ");
-        }
-        else {
-            $content  = $output;
-            $minified = 1;
-        }
-    }
-
     return $content;
 }
 
+sub Filter {
+    my ( $self, $content ) = @_;
+    return JavaScript::Minifier::XS::minify($content);
+}
+
 1;
 
