@@ -1,8 +1,13 @@
+# NOTE: This file is deprecated.  Instead of:
+#    use RT::Test::GnuPG ARGS
+# please do:
+#    use RT::Test::Crypt GnuPG => 1, ARGS
+
 # BEGIN BPS TAGGED BLOCK {{{
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2018 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -52,6 +57,9 @@ use warnings;
 use Test::More;
 use base qw(RT::Test);
 use File::Temp qw(tempdir);
+use IPC::Run3 'run3';
+use File::Copy;
+use 5.010;
 
 our @EXPORT =
   qw(create_a_ticket update_ticket cleanup_headers set_queue_crypt_options 
@@ -64,6 +72,8 @@ sub import {
     my %args  = @_;
     my $t     = $class->builder;
 
+    RT::Test::plan( skip_all => 'ENV SKIP_GPG_TESTS is set to true.' )
+      if $ENV{'SKIP_GPG_TESTS'};
     RT::Test::plan( skip_all => 'GnuPG required.' )
       unless GnuPG::Interface->require;
     RT::Test::plan( skip_all => 'gpg executable is required.' )
@@ -94,7 +104,17 @@ sub bootstrap_more_config {
         'no-permission-warning' => undef,
         $args->{gnupg_options} ? %{ $args->{gnupg_options} } : (),
     );
-    $gnupg_options{homedir} ||= scalar tempdir( CLEANUP => 1 );
+    $gnupg_options{homedir} ||= new_homedir();
+
+    my $conf = File::Spec->catfile( $gnupg_options{homedir}, 'gpg.conf' );
+    if ( gnupg_version() >= 2 ) {
+        open my $fh, '>', $conf or die $!;
+        print $fh "pinentry-mode loopback\n";
+        close $fh;
+    }
+    else {
+        unlink $conf if -e $conf;
+    }
 
     use Data::Dumper;
     local $Data::Dumper::Terse = 1; # "{...}" instead of "$VAR1 = {...};"
@@ -132,7 +152,7 @@ sub create_a_ticket {
         }
     }
 
-    $m->submit;
+    $m->click('SubmitTicket');
     is $m->status, 200, "request successful";
 
     $m->content_lacks("unable to sign outgoing email messages");
@@ -363,3 +383,40 @@ sub create_and_test_outgoing_emails {
         }
     }
 }
+
+sub gnupg_version {
+    GnuPG::Interface->require or return;
+    require version;
+    state $gnupg_version = version->parse(GnuPG::Interface->new->version);
+}
+
+sub new_homedir {
+    my $source = shift;
+    return if $ENV{'SKIP_GPG_TESTS'} || !RT::Test->find_executable('gpg') || !GnuPG::Interface->require;
+    my $dir = tempdir();
+
+    if ($source) {
+        opendir my $dh, $source or die $!;
+        for my $file ( grep {/\.gpg$/} readdir $dh ) {
+            copy( File::Spec->catfile( $source, $file ), File::Spec->catfile( $dir, $file ) ) or die $!;
+        }
+        closedir $dh;
+        if ( gnupg_version() >= 2 ) {
+            # Do the data migration
+            run3( [ 'gpg', '--homedir', $dir, '--list-secret-keys' ], \undef, \undef, \undef );
+        }
+    }
+
+    return $dir;
+}
+
+END {
+    if ( RT::Test->builder()->has_plan && gnupg_version() >= 2 ) {
+        system( 'gpgconf', '--homedir', RT->Config->Get('GnuPGOptions')->{homedir}, '--quiet', '--kill', 'gpg-agent' )
+            && warn $!;
+    }
+
+    diag "RT::Test::GnuPG is deprecated; please use RT::Test::Crypt GnuPG => 1 instead";
+}
+
+1;
